/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;

import com.tinkerforge.BrickSilentStepper;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Silent Stepper Brick.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickSilentStepperWrapper extends BrickSilentStepper implements DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickSilentStepper.DEVICE_DISPLAY_NAME, "bricksilentstepper", BrickSilentStepper.DEVICE_IDENTIFIER, BrickSilentStepperWrapper.class, BrickSilentStepperActions.class, "2.0.4", false);

    public BrickSilentStepperWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickSilentStepperWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickSilentStepperConfig cfg = (BrickSilentStepperConfig) config.as(BrickSilentStepperConfig.class);
        this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperCurrentVelocity", transformBrickSilentStepperCurrentVelocityCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperCurrentPosition", transformBrickSilentStepperCurrentPositionCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperRemainingSteps", transformBrickSilentStepperRemainingStepsCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperStackVoltage", transformBrickSilentStepperStackVoltageCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperExternalVoltage", transformBrickSilentStepperExternalVoltageCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addAllDataListener(this.reg((currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption) -> {if(true) {updateStateFn.accept("BrickSilentStepperCurrentConsumption", transformBrickSilentStepperCurrentConsumptionCallback0(currentVelocity, currentPosition, remainingSteps, stackVoltage, externalVoltage, currentConsumption, cfg));}}, this::removeAllDataListener));
		this.addNewStateListener(this.reg((stateNew, statePrevious) -> {if(true) {updateStateFn.accept("BrickSilentStepperState", transformBrickSilentStepperStateCallback0(stateNew, statePrevious, cfg));}}, this::removeNewStateListener));
		this.addNewStateListener(this.reg((stateNew, statePrevious) -> {if(true) {updateStateFn.accept("BrickSilentStepperPreviousState", transformBrickSilentStepperPreviousStateCallback0(stateNew, statePrevious, cfg));}}, this::removeNewStateListener));
		this.addPositionReachedListener(this.reg((position) -> {if(true) {triggerChannelFn.accept("BrickSilentStepperPositionReached", transformBrickSilentStepperPositionReachedCallback0(position, cfg));}}, this::removePositionReachedListener));
		this.addUnderVoltageListener(this.reg((voltage) -> {if(true) {triggerChannelFn.accept("BrickSilentStepperUnterVoltage", transformBrickSilentStepperUnterVoltageCallback0(voltage, cfg));}}, this::removeUnderVoltageListener));
        this.setAllDataPeriod(cfg.allDataUpdateInterval);
		    this.setMinimumVoltage((int)(cfg.minimumVoltage.doubleValue() * 1000.0));
		if(cfg.statusLEDConfig) {
		                this.enableStatusLED();
		            } else {
		                this.disableStatusLED();
		            }
		this.setSPITFPBaudrateConfig(cfg.spitfpEnableDynamicBaudrate, cfg.spitfpMinimumDynamicBaudrate);
		this.setSPITFPBaudrate('a', cfg.spitfpBaudratePortA);
		            this.setSPITFPBaudrate('b', cfg.spitfpBaudratePortB);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickSilentStepperCurrentVelocity");
		result.add("BrickSilentStepperCurrentPosition");
		result.add("BrickSilentStepperRemainingSteps");
		result.add("BrickSilentStepperStackVoltage");
		result.add("BrickSilentStepperExternalVoltage");
		result.add("BrickSilentStepperCurrentConsumption");
		result.add("BrickSilentStepperState");
		result.add("BrickSilentStepperPreviousState");
		result.add("BrickSilentStepperPositionReached");
		result.add("BrickSilentStepperUnterVoltage");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickSilentStepperCurrentVelocity":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentVelocity"), "Velocity", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperCurrentVelocity")).withDescription("The current velocity of the stepper motor in steps per second.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperCurrentPosition":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentPosition"), "Position", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperCurrentPosition")).withDescription("The current position of the stepper motor in steps. On startup the position is 0.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperRemainingSteps":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperRemainingSteps"), "Remaining Steps", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperRemainingSteps")).withDescription("The remaining steps of the last call of the setSteps() action.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperStackVoltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperStackVoltage"), "Stack Voltage", "Number:ElectricPotential").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperStackVoltage")).withDescription("The stack input voltage in V. The stack input voltage is the voltage that is supplied via the stack, i.e. it is given by a Step-Down or Step-Up Power Supply.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperExternalVoltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperExternalVoltage"), "External Voltage", "Number:ElectricPotential").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperExternalVoltage")).withDescription("The external input voltage in mV. The external input voltage is given via the black power input connector on the Silent Stepper Brick.<br/><br/>If there is an external input voltage and a stack input voltage, the motor will be driven by the external input voltage. If there is only a stack voltage present, the motor will be driven by this voltage.<br/><br/><b>Warning: This means, if you have a high stack voltage and a low external voltage, the motor will be driven with the low external voltage. If you then remove the external connection, it will immediately be driven by the high stack voltage</b>")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperCurrentConsumption":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentConsumption"), "Current Consumption", "Number:ElectricCurrent").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperCurrentConsumption")).withDescription("The current consumption of the motor in A.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickSilentStepperState":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickSilentStepperState"), "State", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickSilentStepperState")).withDescription("State of the brick: <ul><li>Stop = 1</li><li>Acceleration = 2</li><li>Run = 3</li><li>Deacceleration = 4</li><li>Direction change to forward = 5</li><li>Direction change to backward = 6</li></ul>")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%d").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Silent Stepper Brick").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Silently drives one bipolar stepper motor with up to 46V and 1.6A per phase").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickSilentStepperCurrentVelocity", new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentVelocity")).withLabel("Velocity").build(), new ChannelDefinitionBuilder("BrickSilentStepperCurrentPosition", new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentPosition")).withLabel("Position").build(), new ChannelDefinitionBuilder("BrickSilentStepperRemainingSteps", new ChannelTypeUID("tinkerforge", "BrickSilentStepperRemainingSteps")).withLabel("Remaining Steps").build(), new ChannelDefinitionBuilder("BrickSilentStepperStackVoltage", new ChannelTypeUID("tinkerforge", "BrickSilentStepperStackVoltage")).withLabel("Stack Voltage").build(), new ChannelDefinitionBuilder("BrickSilentStepperExternalVoltage", new ChannelTypeUID("tinkerforge", "BrickSilentStepperExternalVoltage")).withLabel("External Voltage").build(), new ChannelDefinitionBuilder("BrickSilentStepperCurrentConsumption", new ChannelTypeUID("tinkerforge", "BrickSilentStepperCurrentConsumption")).withLabel("Current Consumption").build(), new ChannelDefinitionBuilder("BrickSilentStepperState", new ChannelTypeUID("tinkerforge", "BrickSilentStepperState")).withLabel("State").build(), new ChannelDefinitionBuilder("BrickSilentStepperPreviousState", new ChannelTypeUID("tinkerforge", "BrickSilentStepperState")).withLabel("Previous State").build(), new ChannelDefinitionBuilder("BrickSilentStepperPositionReached", new ChannelTypeUID("system", "trigger")).withLabel("Position Reached").withDescription("This channel is triggered when a position set by the setSteps or setTargetPosition action is reached.<br/><br/>Note<br/><br/>Since we can't get any feedback from the stepper motor, this only works if the acceleration (see setSpeedRamping()) is set smaller or equal to the maximum acceleration of the motor. Otherwise the motor will lag behind the control value and the listener will be triggered too early.").build(), new ChannelDefinitionBuilder("BrickSilentStepperUnterVoltage", new ChannelTypeUID("system", "trigger")).withLabel("Unter Voltage").withDescription("This channel is triggered when the input voltage drops below the configured minimum voltage.").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:bricksilentstepper":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("allDataUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for all data in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("All Data Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("minimumVoltage", Type.DECIMAL).withDefault("8").withDescription("The minimum voltage in V, below which the Unter Voltage channel is triggered. The minimum possible value that works with the Silent Stepper Brick is 8V. You can use this function to detect the discharge of a battery that is used to drive the stepper motor. If you have a fixed power supply, you likely do not need this functionality. The default value is 8V.").withLabel("Minimum Voltage").withUnit("V").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(65.535)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.BOOLEAN).withDefault("true").withDescription("The status LED is the blue LED next to the USB connector. If enabled is is on and it flickers if data is transfered. If disabled it is always off.").withLabel("Status LED Configuration").build(), ConfigDescriptionParameterBuilder.create("spitfpEnableDynamicBaudrate", Type.BOOLEAN).withDefault("true").withDescription("The SPITF protocol can be used with a dynamic baudrate. If the dynamic baudrate is enabled, the Brick will try to adapt the baudrate for the communication between Bricks and Bricklets according to the amount of data that is transferred.<br/><br/>The baudrate will be increased exponentially if lots of data is sent/received and decreased linearly if little data is sent/received.<br/><br/>This lowers the baudrate in applications where little data is transferred (e.g. a weather station) and increases the robustness. If there is lots of data to transfer (e.g. Thermal Imaging Bricklet) it automatically increases the baudrate as needed.<br/><br/>In cases where some data has to transferred as fast as possible every few seconds (e.g. RS485 Bricklet with a high baudrate but small payload) you may want to turn the dynamic baudrate off to get the highest possible performance.<br/><br/>The maximum value of the baudrate can be set per port. If the dynamic baudrate is disabled, the maximum baudrate will be used statically.").withLabel("SPITFP Enable Dynamic Baudrate").withAdvanced(true).build(), ConfigDescriptionParameterBuilder.create("spitfpMinimumDynamicBaudrate", Type.INTEGER).withDefault("400000").withDescription("See SPITFP Enable Dynamic Baudrate").withLabel("SPITFP Minimum Dynamic Baudrate").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("spitfpBaudratePortA", Type.INTEGER).withDefault("1400000").withDescription("The baudrate for Bricklet port A.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in your applications we recommend to not change the baudrate.").withLabel("(Maximum) SPITFP Baudrate Port A").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("spitfpBaudratePortB", Type.INTEGER).withDefault("1400000").withDescription("The baudrate for Bricklet port B.<br/><br/>If you want to increase the throughput of Bricklets you can increase the baudrate. If you get a high error count because of high interference you can decrease the baudrate.<br/><br/>If the dynamic baudrate feature is enabled, this is the maximum baudrate.<br/><br/>Regulatory testing is done with the default baudrate. If CE compatibility or similar is necessary in your applications we recommend to not change the baudrate.").withLabel("(Maximum) SPITFP Baudrate Port B").withAdvanced(true).withMinimum(BigDecimal.valueOf(400000)).withMaximum(BigDecimal.valueOf(2000000)).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickSilentStepperCurrentVelocity":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperCurrentPosition":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperRemainingSteps":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperStackVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperExternalVoltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperCurrentConsumption":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickSilentStepperState":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickSilentStepperConfig cfg = (BrickSilentStepperConfig) config.as(BrickSilentStepperConfig.class);
switch(channel) {
            case "BrickSilentStepperCurrentVelocity":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperCurrentVelocityGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperCurrentPosition":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperCurrentPositionGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperRemainingSteps":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperRemainingStepsGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperStackVoltage":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperStackVoltageGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperExternalVoltage":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperExternalVoltageGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperCurrentConsumption":
                   {
            BrickSilentStepper.AllData value = this.getAllData();
            if(true)
                updateStateFn.accept(channel, transformBrickSilentStepperCurrentConsumptionGetter0(value, cfg));
        }
                   return;
            case "BrickSilentStepperState":
               return;
            case "BrickSilentStepperPreviousState":
               return;
            case "BrickSilentStepperPositionReached":
               return;
            case "BrickSilentStepperUnterVoltage":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentVelocityCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new DecimalType(currentVelocity);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentPositionCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new DecimalType(currentPosition);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperRemainingStepsCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new DecimalType(remainingSteps);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperStackVoltageCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(stackVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperExternalVoltageCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(externalVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentConsumptionCallback0(int currentVelocity, int currentPosition, int remainingSteps, int stackVoltage, int externalVoltage, int currentConsumption, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(currentConsumption / 1000.0, SmartHomeUnits.AMPERE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperStateCallback0(short stateNew, short statePrevious, BrickSilentStepperConfig cfg) {
        return new DecimalType(stateNew);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperPreviousStateCallback0(short stateNew, short statePrevious, BrickSilentStepperConfig cfg) {
        return new DecimalType(statePrevious);
    }
	    private String transformBrickSilentStepperPositionReachedCallback0(int position, BrickSilentStepperConfig cfg) {
        return "";
    }
	    private String transformBrickSilentStepperUnterVoltageCallback0(int voltage, BrickSilentStepperConfig cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentVelocityGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new DecimalType(value.currentVelocity);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentPositionGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new DecimalType(value.currentPosition);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperRemainingStepsGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new DecimalType(value.remainingSteps);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperStackVoltageGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(value.stackVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperExternalVoltageGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(value.externalVoltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickSilentStepperCurrentConsumptionGetter0(BrickSilentStepper.AllData value, BrickSilentStepperConfig cfg) {
        return new QuantityType<>(value.currentConsumption / 1000.0, SmartHomeUnits.AMPERE);
    }

}