/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletAnalogOutV2;

/**
 * Passes through actions of the Analog Out Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutV2Actions implements ThingActions, IBrickletAnalogOutV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletAnalogOutV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletAnalogOutV2Actions.class.getName())) {
            if (actions instanceof IBrickletAnalogOutV2Actions) {
                return (IBrickletAnalogOutV2Actions) actions;
            } else {
                return (IBrickletAnalogOutV2Actions) Proxy.newProxyInstance(IBrickletAnalogOutV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletAnalogOutV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletAnalogOutV2Actions");
    }

        @Override
    @RuleAction(label = "Set Output Voltage")
    public void brickletAnalogOutV2SetOutputVoltage(
            @ActionInput(name = "voltage") int voltage) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletAnalogOutV2Wrapper dev = ((@Nullable BrickletAnalogOutV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setOutputVoltage(voltage);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletAnalogOutV2OutputVoltage")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletAnalogOutV2OutputVoltage"), RefreshType.REFRESH);
}
    }

    public static void brickletAnalogOutV2SetOutputVoltage(@Nullable ThingActions actions, int voltage) throws TinkerforgeException {
        invokeMethodOf(actions).brickletAnalogOutV2SetOutputVoltage(voltage);
    }


    @Override
    @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV2GetOutputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletAnalogOutV2Wrapper dev = ((@Nullable BrickletAnalogOutV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getOutputVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetOutputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletAnalogOutV2GetOutputVoltage();
    }


    @Override
    @RuleAction(label = "Get Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletAnalogOutV2GetInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletAnalogOutV2Wrapper dev = ((@Nullable BrickletAnalogOutV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getInputVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletAnalogOutV2GetInputVoltage();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletAnalogOutV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletAnalogOutV2Wrapper dev = ((@Nullable BrickletAnalogOutV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletAnalogOutV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletAnalogOutV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletAnalogOutV2GetIdentity();
    }
}
