/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletCurrent25;

/**
 * Passes through actions of the Current25 Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCurrent25Actions implements ThingActions, IBrickletCurrent25Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletCurrent25Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletCurrent25Actions.class.getName())) {
            if (actions instanceof IBrickletCurrent25Actions) {
                return (IBrickletCurrent25Actions) actions;
            } else {
                return (IBrickletCurrent25Actions) Proxy.newProxyInstance(IBrickletCurrent25Actions.class.getClassLoader(),
                        new Class[] { IBrickletCurrent25Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletCurrent25Actions");
    }

    
    @Override
    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="short")
           Map<String, Object> brickletCurrent25GetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCurrent25Wrapper dev = ((@Nullable BrickletCurrent25Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        short value = dev.getCurrent();
        
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent25GetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCurrent25GetCurrent();
    }


    @Override
    @RuleAction(label = "Is Over Current")
    public @ActionOutput(name = "over", type="boolean")
           Map<String, Object> brickletCurrent25IsOverCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCurrent25Wrapper dev = ((@Nullable BrickletCurrent25Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean value = dev.isOverCurrent();
        
        result.put("over", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent25IsOverCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCurrent25IsOverCurrent();
    }


    @Override
    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletCurrent25GetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCurrent25Wrapper dev = ((@Nullable BrickletCurrent25Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getAnalogValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletCurrent25GetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCurrent25GetAnalogValue();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletCurrent25GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletCurrent25Wrapper dev = ((@Nullable BrickletCurrent25Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletCurrent25.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCurrent25GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletCurrent25GetIdentity();
    }
}
