/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletDualButtonV2;

/**
 * Passes through actions of the Dual Button Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualButtonV2Actions implements ThingActions, IBrickletDualButtonV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletDualButtonV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletDualButtonV2Actions.class.getName())) {
            if (actions instanceof IBrickletDualButtonV2Actions) {
                return (IBrickletDualButtonV2Actions) actions;
            } else {
                return (IBrickletDualButtonV2Actions) Proxy.newProxyInstance(IBrickletDualButtonV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletDualButtonV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletDualButtonV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get LED State")
    public @ActionOutput(name = "ledL", type="int")
           @ActionOutput(name = "ledR", type="int")
           Map<String, Object> brickletDualButtonV2GetLEDState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDualButtonV2.LEDState value = dev.getLEDState();
        
        result.put("ledL", value.ledL);
        result.put("ledR", value.ledR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetLEDState(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetLEDState();
    }


    @Override
    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "buttonL", type="int")
           @ActionOutput(name = "buttonR", type="int")
           Map<String, Object> brickletDualButtonV2GetButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDualButtonV2.ButtonState value = dev.getButtonState();
        
        result.put("buttonL", value.buttonL);
        result.put("buttonR", value.buttonR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetButtonState();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletDualButtonV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletDualButtonV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDualButtonV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletDualButtonV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDualButtonV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletDualButtonV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletDualButtonV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletDualButtonV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDualButtonV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletDualButtonV2Wrapper dev = ((@Nullable BrickletDualButtonV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletDualButtonV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletDualButtonV2GetIdentity();
    }
}
