/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletGPS;

/**
 * Passes through actions of the GPS Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletGPSActions implements ThingActions, IBrickletGPSActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletGPSActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletGPSActions.class.getName())) {
            if (actions instanceof IBrickletGPSActions) {
                return (IBrickletGPSActions) actions;
            } else {
                return (IBrickletGPSActions) Proxy.newProxyInstance(IBrickletGPSActions.class.getClassLoader(),
                        new Class[] { IBrickletGPSActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletGPSActions");
    }

    
    @Override
    @RuleAction(label = "Get Coordinates")
    public @ActionOutput(name = "latitude", type="long")
           @ActionOutput(name = "ns", type="char")
           @ActionOutput(name = "longitude", type="long")
           @ActionOutput(name = "ew", type="char")
           @ActionOutput(name = "pdop", type="int")
           @ActionOutput(name = "hdop", type="int")
           @ActionOutput(name = "vdop", type="int")
           @ActionOutput(name = "epe", type="int")
           Map<String, Object> brickletGPSGetCoordinates(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.Coordinates value = dev.getCoordinates();
        
        result.put("latitude", value.latitude);
        result.put("ns", value.ns);
        result.put("longitude", value.longitude);
        result.put("ew", value.ew);
        result.put("pdop", value.pdop);
        result.put("hdop", value.hdop);
        result.put("vdop", value.vdop);
        result.put("epe", value.epe);
        return result;
    }

    public static Map<String, Object> brickletGPSGetCoordinates(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetCoordinates();
    }


    @Override
    @RuleAction(label = "Get Status")
    public @ActionOutput(name = "fix", type="short")
           @ActionOutput(name = "satellitesView", type="short")
           @ActionOutput(name = "satellitesUsed", type="short")
           Map<String, Object> brickletGPSGetStatus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.Status value = dev.getStatus();
        
        result.put("fix", value.fix);
        result.put("satellitesView", value.satellitesView);
        result.put("satellitesUsed", value.satellitesUsed);
        return result;
    }

    public static Map<String, Object> brickletGPSGetStatus(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetStatus();
    }


    @Override
    @RuleAction(label = "Get Altitude")
    public @ActionOutput(name = "altitude", type="int")
           @ActionOutput(name = "geoidalSeparation", type="int")
           Map<String, Object> brickletGPSGetAltitude(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.Altitude value = dev.getAltitude();
        
        result.put("altitude", value.altitude);
        result.put("geoidalSeparation", value.geoidalSeparation);
        return result;
    }

    public static Map<String, Object> brickletGPSGetAltitude(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetAltitude();
    }


    @Override
    @RuleAction(label = "Get Motion")
    public @ActionOutput(name = "course", type="long")
           @ActionOutput(name = "speed", type="long")
           Map<String, Object> brickletGPSGetMotion(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.Motion value = dev.getMotion();
        
        result.put("course", value.course);
        result.put("speed", value.speed);
        return result;
    }

    public static Map<String, Object> brickletGPSGetMotion(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetMotion();
    }


    @Override
    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "date", type="long")
           @ActionOutput(name = "time", type="long")
           Map<String, Object> brickletGPSGetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.DateTime value = dev.getDateTime();
        
        result.put("date", value.date);
        result.put("time", value.time);
        return result;
    }

    public static Map<String, Object> brickletGPSGetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetDateTime();
    }

    @Override
    @RuleAction(label = "Restart")
    public void brickletGPSRestart(
            @ActionInput(name = "restartType") short restartType) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.restart(restartType);
        
    }

    public static void brickletGPSRestart(@Nullable ThingActions actions, short restartType) throws TinkerforgeException {
        invokeMethodOf(actions).brickletGPSRestart(restartType);
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletGPSGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletGPSWrapper dev = ((@Nullable BrickletGPSWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletGPS.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletGPSGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletGPSGetIdentity();
    }
}
