/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIMUV3;

/**
 * Passes through actions of the IMU Bricklet 3.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIMUV3Actions implements ThingActions, IBrickletIMUV3Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletIMUV3Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletIMUV3Actions.class.getName())) {
            if (actions instanceof IBrickletIMUV3Actions) {
                return (IBrickletIMUV3Actions) actions;
            } else {
                return (IBrickletIMUV3Actions) Proxy.newProxyInstance(IBrickletIMUV3Actions.class.getClassLoader(),
                        new Class[] { IBrickletIMUV3Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletIMUV3Actions");
    }

    
    @Override
    @RuleAction(label = "Get Orientation")
    public @ActionOutput(name = "heading", type="int")
           @ActionOutput(name = "roll", type="int")
           @ActionOutput(name = "pitch", type="int")
           Map<String, Object> brickletIMUV3GetOrientation(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.Orientation value = dev.getOrientation();
        
        result.put("heading", value.heading);
        result.put("roll", value.roll);
        result.put("pitch", value.pitch);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetOrientation(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetOrientation();
    }


    @Override
    @RuleAction(label = "Get Linear Acceleration")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetLinearAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.LinearAcceleration value = dev.getLinearAcceleration();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetLinearAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetLinearAcceleration();
    }


    @Override
    @RuleAction(label = "Get Gravity Vector")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetGravityVector(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.GravityVector value = dev.getGravityVector();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetGravityVector(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetGravityVector();
    }


    @Override
    @RuleAction(label = "Get Quaternion")
    public @ActionOutput(name = "w", type="int")
           @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetQuaternion(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.Quaternion value = dev.getQuaternion();
        
        result.put("w", value.w);
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetQuaternion(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetQuaternion();
    }


    @Override
    @RuleAction(label = "Get All Data")
    public @ActionOutput(name = "acceleration", type="int[]")
           @ActionOutput(name = "magneticField", type="int[]")
           @ActionOutput(name = "angularVelocity", type="int[]")
           @ActionOutput(name = "eulerAngle", type="int[]")
           @ActionOutput(name = "quaternion", type="int[]")
           @ActionOutput(name = "linearAcceleration", type="int[]")
           @ActionOutput(name = "gravityVector", type="int[]")
           @ActionOutput(name = "temperature", type="int")
           @ActionOutput(name = "calibrationStatus", type="int")
           Map<String, Object> brickletIMUV3GetAllData(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.AllData value = dev.getAllData();
        
        result.put("acceleration", value.acceleration);
        result.put("magneticField", value.magneticField);
        result.put("angularVelocity", value.angularVelocity);
        result.put("eulerAngle", value.eulerAngle);
        result.put("quaternion", value.quaternion);
        result.put("linearAcceleration", value.linearAcceleration);
        result.put("gravityVector", value.gravityVector);
        result.put("temperature", value.temperature);
        result.put("calibrationStatus", value.calibrationStatus);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetAllData(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetAllData();
    }


    @Override
    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetAcceleration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.Acceleration value = dev.getAcceleration();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetAcceleration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetAcceleration();
    }


    @Override
    @RuleAction(label = "Get Magnetic Field")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetMagneticField(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.MagneticField value = dev.getMagneticField();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetMagneticField(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetMagneticField();
    }


    @Override
    @RuleAction(label = "Get Angular Velocity")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> brickletIMUV3GetAngularVelocity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.AngularVelocity value = dev.getAngularVelocity();
        
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetAngularVelocity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetAngularVelocity();
    }


    @Override
    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIMUV3GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetTemperature();
    }


    @Override
    @RuleAction(label = "Get Sensor Configuration")
    public @ActionOutput(name = "magnetometerRate", type="int")
           @ActionOutput(name = "gyroscopeRange", type="int")
           @ActionOutput(name = "gyroscopeBandwidth", type="int")
           @ActionOutput(name = "accelerometerRange", type="int")
           @ActionOutput(name = "accelerometerBandwidth", type="int")
           Map<String, Object> brickletIMUV3GetSensorConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.SensorConfiguration value = dev.getSensorConfiguration();
        
        result.put("magnetometerRate", value.magnetometerRate);
        result.put("gyroscopeRange", value.gyroscopeRange);
        result.put("gyroscopeBandwidth", value.gyroscopeBandwidth);
        result.put("accelerometerRange", value.accelerometerRange);
        result.put("accelerometerBandwidth", value.accelerometerBandwidth);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetSensorConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetSensorConfiguration();
    }


    @Override
    @RuleAction(label = "Get Sensor Fusion Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletIMUV3GetSensorFusionMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getSensorFusionMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetSensorFusionMode(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetSensorFusionMode();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIMUV3ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIMUV3ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIMUV3GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIMUV3GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIMUV3GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletIMUV3Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletIMUV3Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletIMUV3Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIMUV3GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIMUV3Wrapper dev = ((@Nullable BrickletIMUV3Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIMUV3.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIMUV3GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIMUV3GetIdentity();
    }
}
