/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialDigitalIn4;

/**
 * Passes through actions of the Industrial Digital In 4 Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDigitalIn4Actions implements ThingActions, IBrickletIndustrialDigitalIn4Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletIndustrialDigitalIn4Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletIndustrialDigitalIn4Actions.class.getName())) {
            if (actions instanceof IBrickletIndustrialDigitalIn4Actions) {
                return (IBrickletIndustrialDigitalIn4Actions) actions;
            } else {
                return (IBrickletIndustrialDigitalIn4Actions) Proxy.newProxyInstance(IBrickletIndustrialDigitalIn4Actions.class.getClassLoader(),
                        new Class[] { IBrickletIndustrialDigitalIn4Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletIndustrialDigitalIn4Actions");
    }

    
    @Override
    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getValue();
        
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4GetValue();
    }


    @Override
    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCount(
            @ActionInput(name = "pin") short pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        Thing t = h.getThing();
        @Nullable BrickletIndustrialDigitalIn4Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getEdgeCount(pin, resetCounter);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount0")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount0"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount1")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount1"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount2")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount2"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount3")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4EdgeCount3"), RefreshType.REFRESH);
}
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCount(@Nullable ThingActions actions, short pin, boolean resetCounter) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4GetEdgeCount(pin, resetCounter);
    }


    @Override
    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="short")
           @ActionOutput(name = "debounce", type="short")
           Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCountConfig(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialDigitalIn4.EdgeCountConfig value = dev.getEdgeCountConfig(pin);
        
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetEdgeCountConfig(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4GetEdgeCountConfig(pin);
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialDigitalIn4.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4GetIdentity();
    }
}
