/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletIndustrialDigitalIn4V2;

/**
 * Passes through actions of the Industrial Digital In 4 Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDigitalIn4V2Actions implements ThingActions, IBrickletIndustrialDigitalIn4V2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletIndustrialDigitalIn4V2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletIndustrialDigitalIn4V2Actions.class.getName())) {
            if (actions instanceof IBrickletIndustrialDigitalIn4V2Actions) {
                return (IBrickletIndustrialDigitalIn4V2Actions) actions;
            } else {
                return (IBrickletIndustrialDigitalIn4V2Actions) Proxy.newProxyInstance(IBrickletIndustrialDigitalIn4V2Actions.class.getClassLoader(),
                        new Class[] { IBrickletIndustrialDigitalIn4V2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletIndustrialDigitalIn4V2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean[] value = dev.getValue();
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetValue(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetValue();
    }


    @Override
    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChannelLEDConfig(channel);
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetChannelLEDConfig(channel);
    }


    @Override
    @RuleAction(label = "Get Edge Count Configuration")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetEdgeCountConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialDigitalIn4V2.EdgeCountConfiguration value = dev.getEdgeCountConfiguration(channel);
        
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetEdgeCountConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetEdgeCountConfiguration(channel);
    }


    @Override
    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetEdgeCount(
            @ActionInput(name = "channel") int channel,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        Thing t = h.getThing();
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getEdgeCount(channel, resetCounter);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount0")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount0"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount1")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount1"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount2")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount2"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount3")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletIndustrialDigitalIn4V2EdgeCount3"), RefreshType.REFRESH);
}
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetEdgeCount(@Nullable ThingActions actions, int channel, boolean resetCounter) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetEdgeCount(channel, resetCounter);
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletIndustrialDigitalIn4V2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialDigitalIn4V2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletIndustrialDigitalIn4V2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletIndustrialDigitalIn4V2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletIndustrialDigitalIn4V2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletIndustrialDigitalIn4V2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletIndustrialDigitalIn4V2Wrapper dev = ((@Nullable BrickletIndustrialDigitalIn4V2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletIndustrialDigitalIn4V2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletIndustrialDigitalIn4V2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletIndustrialDigitalIn4V2GetIdentity();
    }
}
