/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;

import com.tinkerforge.BrickletIndustrialDualACRelay;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Industrial Dual AC Relay Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDualACRelayWrapper extends BrickletIndustrialDualACRelay implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDualACRelay.DEVICE_DISPLAY_NAME, "brickletindustrialdualacrelay", BrickletIndustrialDualACRelay.DEVICE_IDENTIFIER, BrickletIndustrialDualACRelayWrapper.class, BrickletIndustrialDualACRelayActions.class, "2.0.0", true);

    public BrickletIndustrialDualACRelayWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDualACRelayWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDualACRelayConfig cfg = (BrickletIndustrialDualACRelayConfig) config.as(BrickletIndustrialDualACRelayConfig.class);
        this.addMonoflopDoneListener(this.reg((channel, value) -> {if(channel == 0) {updateStateFn.accept("BrickletIndustrialDualACRelayRelay0", transformBrickletIndustrialDualACRelayRelay0Callback0(channel, value, cfg));}}, this::removeMonoflopDoneListener));
		this.addMonoflopDoneListener(this.reg((channel, value) -> {if(channel == 1) {updateStateFn.accept("BrickletIndustrialDualACRelayRelay1", transformBrickletIndustrialDualACRelayRelay1Callback0(channel, value, cfg));}}, this::removeMonoflopDoneListener));
        
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletIndustrialDualACRelayRelayConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDualACRelayRelay0").as(BrickletIndustrialDualACRelayRelayConfig.class);
		this.setChannelLEDConfig(0, channelCfg.channelLEDConfiguration);
		}
		{
		BrickletIndustrialDualACRelayRelayConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDualACRelayRelay1").as(BrickletIndustrialDualACRelayRelayConfig.class);
		this.setChannelLEDConfig(1, channelCfg.channelLEDConfiguration);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDualACRelayRelay0");
		result.add("BrickletIndustrialDualACRelayRelay1");
		result.add("BrickletIndustrialDualACRelayMonoflopRelay0");
		result.add("BrickletIndustrialDualACRelayMonoflopRelay1");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletIndustrialDualACRelayRelay":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayRelay"), "NOT USED", "Switch").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialDualACRelayRelay")).withDescription("NOT USED").build();
            case "BrickletIndustrialDualACRelayMonoflop":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayMonoflop"), "NOT USED", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialDualACRelayMonoflop")).withDescription("Triggers a monoflop as configured")
.withCommandDescription(CommandDescriptionBuilder.create().withCommandOption(new CommandOption("TRIGGER", "Trigger")).build()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Dual AC Relay Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Two relays to switch AC devices").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIndustrialDualACRelayRelay0", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayRelay")).withLabel("Relay 0").withDescription("Switches Relay 0. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.").build(), new ChannelDefinitionBuilder("BrickletIndustrialDualACRelayRelay1", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayRelay")).withLabel("Relay 1").withDescription("Switches Relay 1. A running monoflop timer for this relay will be aborted if the relay is toggled by this channel.").build(), new ChannelDefinitionBuilder("BrickletIndustrialDualACRelayMonoflopRelay0", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayMonoflop")).withLabel("Monoflop Relay 0").build(), new ChannelDefinitionBuilder("BrickletIndustrialDualACRelayMonoflopRelay1", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualACRelayMonoflop")).withLabel("Monoflop Relay 1").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdualacrelay":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletIndustrialDualACRelayRelay":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("channelLEDConfiguration", Type.INTEGER).withDefault("3").withDescription("Each channel has a corresponding LED. You can turn the LED off, on or show a heartbeat. You can also set the LED to Channel Status. In this mode the LED is on if the channel is high and off otherwise.").withLabel("Channel LED").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Channel Status"))).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletIndustrialDualACRelayMonoflop":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("monoflopDuration", Type.INTEGER).withDefault("1000").withDescription("The time that the relay should hold the configured value.").withLabel("Monoflop Duration").withUnit("ms").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("monoflopValue", Type.BOOLEAN).withDefault("true").withDescription("The desired value of the specified channel. Activated means relay closed and Deactivated means relay open.").withLabel("Monoflop Value").withAdvanced(true).build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDualACRelayConfig cfg = (BrickletIndustrialDualACRelayConfig) config.as(BrickletIndustrialDualACRelayConfig.class);
switch(channel) {
            case "BrickletIndustrialDualACRelayRelay0":
                   {
            BrickletIndustrialDualACRelay.Value value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualACRelayRelay0Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDualACRelayRelay1":
                   {
            BrickletIndustrialDualACRelay.Value value = this.getValue();
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualACRelayRelay1Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDualACRelayMonoflopRelay0":
                   {
            BrickletIndustrialDualACRelay.Monoflop value = this.getMonoflop(0);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualACRelayMonoflopRelay0Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDualACRelayMonoflopRelay1":
                   {
            BrickletIndustrialDualACRelay.Monoflop value = this.getMonoflop(1);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualACRelayMonoflopRelay1Getter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = new ArrayList<SetterRefresh>();
        
        switch(channel) {
            case "BrickletIndustrialDualACRelayRelay0":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
this.setSelectedValue(0, cmd == OnOffType.ON);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletIndustrialDualACRelayRelay1":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
this.setSelectedValue(1, cmd == OnOffType.ON);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletIndustrialDualACRelayMonoflopRelay0":
                if (command instanceof StringType) {
                    BrickletIndustrialDualACRelayMonoflopConfig channelCfg = channelConfig.as(BrickletIndustrialDualACRelayMonoflopConfig.class);
this.setMonoflop(0, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.", command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDualACRelayRelay0", 0));
                break;
            case "BrickletIndustrialDualACRelayMonoflopRelay1":
                if (command instanceof StringType) {
                    BrickletIndustrialDualACRelayMonoflopConfig channelCfg = channelConfig.as(BrickletIndustrialDualACRelayMonoflopConfig.class);
this.setMonoflop(1, channelCfg.monoflopValue.booleanValue(), channelCfg.monoflopDuration);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.", command.getClass().getName(), channel);
                }
                result.add(new SetterRefresh("BrickletIndustrialDualACRelayRelay1", 0));
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayRelay0Callback0(int channel, boolean value, BrickletIndustrialDualACRelayConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayRelay1Callback0(int channel, boolean value, BrickletIndustrialDualACRelayConfig cfg) {
        return value ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayRelay0Getter0(BrickletIndustrialDualACRelay.Value value, BrickletIndustrialDualACRelayConfig cfg) {
        return value.channel0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayRelay1Getter0(BrickletIndustrialDualACRelay.Value value, BrickletIndustrialDualACRelayConfig cfg) {
        return value.channel1 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayMonoflopRelay0Getter0(BrickletIndustrialDualACRelay.Monoflop value, BrickletIndustrialDualACRelayConfig cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualACRelayMonoflopRelay1Getter0(BrickletIndustrialDualACRelay.Monoflop value, BrickletIndustrialDualACRelayConfig cfg) {
        return value.value ? OnOffType.ON : OnOffType.OFF;
    }

}