/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;

import com.tinkerforge.BrickletIndustrialDualAnalogInV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Industrial Dual Analog In Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletIndustrialDualAnalogInV2Wrapper extends BrickletIndustrialDualAnalogInV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletIndustrialDualAnalogInV2.DEVICE_DISPLAY_NAME, "brickletindustrialdualanaloginv2", BrickletIndustrialDualAnalogInV2.DEVICE_IDENTIFIER, BrickletIndustrialDualAnalogInV2Wrapper.class, BrickletIndustrialDualAnalogInV2Actions.class, "2.0.0", true);

    public BrickletIndustrialDualAnalogInV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletIndustrialDualAnalogInV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletIndustrialDualAnalogInV2Config cfg = (BrickletIndustrialDualAnalogInV2Config) config.as(BrickletIndustrialDualAnalogInV2Config.class);
        this.addVoltageListener(this.reg((channel, voltage) -> {if(channel == 0) {updateStateFn.accept("BrickletIndustrialDualAnalogInV2VoltageChannel0", transformBrickletIndustrialDualAnalogInV2VoltageChannel0Callback0(channel, voltage, cfg));}}, this::removeVoltageListener));
		this.addVoltageListener(this.reg((channel, voltage) -> {if(channel == 1) {updateStateFn.accept("BrickletIndustrialDualAnalogInV2VoltageChannel1", transformBrickletIndustrialDualAnalogInV2VoltageChannel1Callback0(channel, voltage, cfg));}}, this::removeVoltageListener));
        this.setSampleRate(cfg.sampleRate);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
		{
		BrickletIndustrialDualAnalogInV2VoltageConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDualAnalogInV2VoltageChannel0").as(BrickletIndustrialDualAnalogInV2VoltageConfig.class);
		this.setVoltageCallbackConfiguration(0, channelCfg.updateInterval, true, 'x', 0, 0);
		this.setChannelLEDConfig(0, channelCfg.ledConfig);
		this.setChannelLEDStatusConfig(0, channelCfg.ledStatusMinimum, channelCfg.ledStatusMaximum, channelCfg.ledStatusMode);
		}
		{
		BrickletIndustrialDualAnalogInV2VoltageConfig channelCfg = getChannelConfigFn.apply("BrickletIndustrialDualAnalogInV2VoltageChannel1").as(BrickletIndustrialDualAnalogInV2VoltageConfig.class);
		this.setVoltageCallbackConfiguration(1, channelCfg.updateInterval, true, 'x', 0, 0);
		this.setChannelLEDConfig(1, channelCfg.ledConfig);
		this.setChannelLEDStatusConfig(1, channelCfg.ledStatusMinimum, channelCfg.ledStatusMaximum, channelCfg.ledStatusMode);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setVoltageCallbackConfiguration(0, 0, true, 'x', 0, 0);
		this.setVoltageCallbackConfiguration(1, 0, true, 'x', 0, 0);
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletIndustrialDualAnalogInV2VoltageChannel0");
		result.add("BrickletIndustrialDualAnalogInV2VoltageChannel1");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletIndustrialDualAnalogInV2Voltage":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInV2Voltage"), "NOT USED", "Number:ElectricPotential").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletIndustrialDualAnalogInV2Voltage")).withDescription("The measured voltage")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-35)).withMaximum(BigDecimal.valueOf(35)).withStep(BigDecimal.valueOf(0.001)).withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Industrial Dual Analog In Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Measures two DC voltages between -35V and +35V with 24bit resolution each").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletIndustrialDualAnalogInV2VoltageChannel0", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInV2Voltage")).withLabel("Voltage Channel 0").build(), new ChannelDefinitionBuilder("BrickletIndustrialDualAnalogInV2VoltageChannel1", new ChannelTypeUID("tinkerforge", "BrickletIndustrialDualAnalogInV2Voltage")).withLabel("Voltage Channel 1").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletindustrialdualanaloginv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("sampleRate", Type.INTEGER).withDefault("6").withDescription("The voltage measurement sample rate. Decreasing the sample rate will also decrease the noise on the data.").withLabel("Sample Rate").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "976 SPS"), new ParameterOption("1", "488 SPS"), new ParameterOption("2", "244 SPS"), new ParameterOption("3", "122 SPS"), new ParameterOption("4", "61 SPS"), new ParameterOption("5", "4 SPS"), new ParameterOption("6", "2 SPS"), new ParameterOption("7", "1 SPS"))).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletIndustrialDualAnalogInV2Voltage":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build(), ConfigDescriptionParameterBuilder.create("ledConfig", Type.INTEGER).withDefault("3").withDescription("Each channel has a corresponding LED. You can turn the LED off, on or show a heartbeat. You can also set the LED to Show Channel Status. In this mode the LED can either be turned on with a pre-defined threshold or the intensity of the LED can change with the measured value.").withLabel("LED Configuration").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Channel Status"))).build(), ConfigDescriptionParameterBuilder.create("ledStatusMode", Type.INTEGER).withDefault("1").withDescription("For each channel you can choose between threshold and intensity mode.<br/><br/>In threshold mode you can define a positive or a negative threshold.<br/>For a positive threshold set the \"min\" parameter to the threshold value in mV<br/>above which the LED should turn on and set the \"max\" parameter to 0. Example:<br/>If you set a positive threshold of 10V, the LED will turn on as soon as the<br/>voltage exceeds 10V and turn off again if it goes below 10V.<br/>For a negative threshold set the \"max\" parameter to the threshold value in mV<br/>below which the LED should turn on and set the \"min\" parameter to 0. Example:<br/>If you set a negative threshold of 10V, the LED will turn on as soon as the<br/>voltage goes below 10V and the LED will turn off when the voltage exceeds 10V.<br/><br/>In intensity mode you can define a range in mV that is used to scale the brightness<br/>of the LED. Example with min=4V, max=20V: The LED is off at 4V, on at 20V<br/>and the brightness is linearly scaled between the values 4V and 20V. If the<br/>min value is greater than the max value, the LED brightness is scaled the other<br/>way around.").withLabel("LED Status Mode").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Threshold"), new ParameterOption("1", "Intensity"))).build(), ConfigDescriptionParameterBuilder.create("ledStatusMinimum", Type.INTEGER).withDefault("0").withDescription("See LED Status Mode for further explaination.").withLabel("LED Status Minimum").withUnit("V").withMinimum(BigDecimal.valueOf(-35)).withMaximum(BigDecimal.valueOf(35)).build(), ConfigDescriptionParameterBuilder.create("ledStatusMaximum", Type.INTEGER).withDefault("10").withDescription("See LED Status Mode for further explaination.").withLabel("LED Status Maximum").withUnit("V").withMinimum(BigDecimal.valueOf(-35)).withMaximum(BigDecimal.valueOf(35)).build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletIndustrialDualAnalogInV2Config cfg = (BrickletIndustrialDualAnalogInV2Config) config.as(BrickletIndustrialDualAnalogInV2Config.class);
switch(channel) {
            case "BrickletIndustrialDualAnalogInV2VoltageChannel0":
                   {
            int value = this.getVoltage(0);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualAnalogInV2VoltageChannel0Getter0(value, cfg));
        }
                   return;
            case "BrickletIndustrialDualAnalogInV2VoltageChannel1":
                   {
            int value = this.getVoltage(1);
            if(true)
                updateStateFn.accept(channel, transformBrickletIndustrialDualAnalogInV2VoltageChannel1Getter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInV2VoltageChannel0Callback0(int channel, int voltage, BrickletIndustrialDualAnalogInV2Config cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInV2VoltageChannel1Callback0(int channel, int voltage, BrickletIndustrialDualAnalogInV2Config cfg) {
        return new QuantityType<>(voltage / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInV2VoltageChannel0Getter0(int value, BrickletIndustrialDualAnalogInV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletIndustrialDualAnalogInV2VoltageChannel1Getter0(int value, BrickletIndustrialDualAnalogInV2Config cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.VOLT);
    }

}