/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletLEDStripV2;

/**
 * Passes through actions of the LED Strip Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripV2Actions implements ThingActions, IBrickletLEDStripV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletLEDStripV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletLEDStripV2Actions.class.getName())) {
            if (actions instanceof IBrickletLEDStripV2Actions) {
                return (IBrickletLEDStripV2Actions) actions;
            } else {
                return (IBrickletLEDStripV2Actions) Proxy.newProxyInstance(IBrickletLEDStripV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletLEDStripV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletLEDStripV2Actions");
    }

        @Override
    @RuleAction(label = "Set LED Values")
    public void brickletLEDStripV2SetLEDValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "value") int[] value) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setLEDValues(index, value);
        
    }

    public static void brickletLEDStripV2SetLEDValues(@Nullable ThingActions actions, int index, int[] value) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLEDStripV2SetLEDValues(index, value);
    }


    @Override
    @RuleAction(label = "Get LED Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> brickletLEDStripV2GetLEDValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int[] value = dev.getLEDValues(index, length);
        
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetLEDValues(@Nullable ThingActions actions, int index, int length) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetLEDValues(index, length);
    }


    @Override
    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> brickletLEDStripV2GetFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getFrameDuration();
        
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetFrameDuration();
    }


    @Override
    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletLEDStripV2GetSupplyVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getSupplyVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetSupplyVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetSupplyVoltage();
    }


    @Override
    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type="long")
           Map<String, Object> brickletLEDStripV2GetClockFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getClockFrequency();
        
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetClockFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetClockFrequency();
    }


    @Override
    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type="int")
           Map<String, Object> brickletLEDStripV2GetChipType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipType();
        
        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetChipType(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetChipType();
    }


    @Override
    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type="int")
           Map<String, Object> brickletLEDStripV2GetChannelMapping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChannelMapping();
        
        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetChannelMapping(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetChannelMapping();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletLEDStripV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletLEDStripV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletLEDStripV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletLEDStripV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLEDStripV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletLEDStripV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletLEDStripV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLEDStripV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLEDStripV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLEDStripV2Wrapper dev = ((@Nullable BrickletLEDStripV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLEDStripV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLEDStripV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLEDStripV2GetIdentity();
    }
}
