/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import org.eclipse.smarthome.core.library.types.StringType;
import org.eclipse.smarthome.core.library.types.HSBType;

import com.tinkerforge.BrickletLEDStrip;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the LED Strip Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletLEDStripWrapper extends BrickletLEDStrip implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletLEDStrip.DEVICE_DISPLAY_NAME, "brickletledstrip", BrickletLEDStrip.DEVICE_IDENTIFIER, BrickletLEDStripWrapper.class, BrickletLEDStripActions.class, "2.0.6", false);

    public BrickletLEDStripWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletLEDStripWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletLEDStripConfig cfg = (BrickletLEDStripConfig) config.as(BrickletLEDStripConfig.class);
        if(cfg.enableFrameStartedChannel) {
this.addFrameRenderedListener(this.reg((length) -> {if(true) {triggerChannelFn.accept("BrickletLEDStripFrameStarted", transformBrickletLEDStripFrameStartedCallback0(length, cfg));}}, this::removeFrameRenderedListener));}
        this.setChipType(cfg.chipType);
		    this.setChannelMapping(cfg.channelMapping.shortValue());
		    this.setFrameDuration(cfg.frameDuration);
		    this.setClockFrequency(cfg.clockFrequency.longValue());
		if (cfg.enableFrameStartedChannel) {
		this.enableFrameRenderedCallback();
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        BrickletLEDStripConfig cfg = (BrickletLEDStripConfig) config.as(BrickletLEDStripConfig.class);
        if(cfg.enableFrameStartedChannel) {
this.disableFrameRenderedCallback();}
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        BrickletLEDStripConfig cfg = (BrickletLEDStripConfig) config.as(BrickletLEDStripConfig.class);
List<String> result = new ArrayList<String>();
        if (cfg.enableFrameStartedChannel) {
                result.add("BrickletLEDStripFrameStarted");
            }
		result.add("BrickletLEDStripLEDValues");
		result.add("BrickletLEDStripAllLEDs");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletLEDStripLEDValues":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletLEDStripLEDValues"), "LED Values", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLEDStripLEDValues")).withDescription("The RGB(W) values for the LEDs. Changes will be applied the next time the Frame Started Channel triggers.<br/><br/>Command format is a ','-separated list of integers. The first integer is the index of the first LED (not the first value!) to set, additional integers are the values to set. If the channel mapping has 3 colors, you need to give the data in the sequence R,G,B,R,G,B,R,G,B,... if the channel mapping has 4 colors you need to give data in the sequence R,G,B,W,R,G,B,W,R,G,B,W...<br/><br/>The data is double buffered and the colors will be transfered to the LEDs when the next frame duration ends. You can set at most 960 values (320 RGB or 240 RGBW LEDs).<br/><br/> For example sending 2,255,0,0,0,255,0,0,0,255 will set the LED 2 to red, LED 3 to green and LED 4 to blue.").build();
            case "BrickletLEDStripAllLEDs":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletLEDStripAllLEDs"), "All LEDs", "Color").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLEDStripAllLEDs")).withDescription("This channel allows you to set a configurable amount of LEDs (up to 240 RGBW LEDs or 320 RGB LEDs) to the same color. If you want more fine-grained control over the LEDs, use the LED Values channel or the actions.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge LED Strip Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Controls up to 320 RGB LEDs").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletLEDStripFrameStarted", new ChannelTypeUID("system", "trigger")).withLabel("Frame Started").withDescription("This channel is triggered directly after a new frame render is started. You should send the data for the next frame directly after this listener was triggered.").build(), new ChannelDefinitionBuilder("BrickletLEDStripLEDValues", new ChannelTypeUID("tinkerforge", "BrickletLEDStripLEDValues")).withLabel("LED Values").build(), new ChannelDefinitionBuilder("BrickletLEDStripAllLEDs", new ChannelTypeUID("tinkerforge", "BrickletLEDStripAllLEDs")).withLabel("All LEDs").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletledstrip":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("chipType", Type.INTEGER).withDefault("2801").withDescription("The type of the LED driver chip. We currently support the chips<br/><br/><ul><li>WS2801</li><li>WS2811</li><li>WS2812 / SK6812 / NeoPixel RGB</li><li>SK6812RGBW / NeoPixel RGBW (Chip Type = WS2812)</li><li>LPD8806</li><li>APA102 / DotStar</li></ul>").withLabel("LED Driver Chip Type").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("2801", "WS2801"), new ParameterOption("2811", "WS2811"), new ParameterOption("2812", "WS2812"), new ParameterOption("8806", "LPD8806"), new ParameterOption("102", "APA102"))).build(), ConfigDescriptionParameterBuilder.create("channelMapping", Type.INTEGER).withDefault("36").withDescription("The channel mapping for the connected LEDs.<br/><br/>If the mapping has 4 colors, the LED Values channel expects 4 values per pixel and if the mapping has 3 colors it expects 3 values per pixel.<br/><br/>The LED Values channel always expects the order RGB(W). The connected LED driver chips might have their 3 or 4 channels in a different order. For example, the WS2801 chips typically use BGR order, then WS2812 chips typically use GRB order and the APA102 chips typically use WBGR order.<br/><br/>The APA102 chips are special. They have three 8-bit channels for RGB and an additional 5-bit channel for the overall brightness of the RGB LED making them 4-channel chips. Internally the brightness channel is the first channel, therefore one of the Wxyz channel mappings should be used. Then the W channel controls the brightness.").withLabel("LED Channel Mapping").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("6", "RGB"), new ParameterOption("9", "RBG"), new ParameterOption("33", "BRG"), new ParameterOption("36", "BGR"), new ParameterOption("18", "GRB"), new ParameterOption("24", "GBR"), new ParameterOption("27", "RGBW"), new ParameterOption("30", "RGWB"), new ParameterOption("39", "RBGW"), new ParameterOption("45", "RBWG"), new ParameterOption("54", "RWGB"), new ParameterOption("57", "RWBG"), new ParameterOption("78", "GRWB"), new ParameterOption("75", "GRBW"), new ParameterOption("108", "GBWR"), new ParameterOption("99", "GBRW"), new ParameterOption("120", "GWBR"), new ParameterOption("114", "GWRB"), new ParameterOption("135", "BRGW"), new ParameterOption("141", "BRWG"), new ParameterOption("147", "BGRW"), new ParameterOption("156", "BGWR"), new ParameterOption("177", "BWRG"), new ParameterOption("180", "BWGR"), new ParameterOption("201", "WRBG"), new ParameterOption("198", "WRGB"), new ParameterOption("216", "WGBR"), new ParameterOption("210", "WGRB"), new ParameterOption("228", "WBGR"), new ParameterOption("225", "WBRG"))).build(), ConfigDescriptionParameterBuilder.create("frameDuration", Type.INTEGER).withDefault("1000").withDescription("The frame duration in milliseconds. This configures how fast the Frame Started Channel will trigger.").withLabel("Frame Duration").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(65535)).build(), ConfigDescriptionParameterBuilder.create("clockFrequency", Type.INTEGER).withDefault("1666666").withDescription("The frequency of the clock in Hz. The Bricklet will choose the nearest achievable frequency, which may be off by a few Hz.<br/><br/>If you have problems with flickering LEDs, they may be bits flipping. You can fix this by either making the connection between the LEDs and the Bricklet shorter or by reducing the frequency.<br/><br/>With a decreasing frequency your maximum frames per second will decrease too.").withLabel("Clock Frequency").withMinimum(BigDecimal.valueOf(10000)).withMaximum(BigDecimal.valueOf(2000000)).build(), ConfigDescriptionParameterBuilder.create("enableFrameStartedChannel", Type.BOOLEAN).withDefault("false").withDescription("Enables the frame started channel. This will result in a lot of spam in the openhab Log!").withLabel("Enable Frame Started Channel").build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletLEDStripLEDValues":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLEDStripAllLEDs":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("ledCount", Type.INTEGER).withDefault("0").withDescription("The number of LEDs to control.").withLabel("LEDs").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(320)).build())).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        switch(channel) {
            case "BrickletLEDStripFrameStarted":
               return;
            case "BrickletLEDStripLEDValues":
               return;
            case "BrickletLEDStripAllLEDs":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        BrickletLEDStripConfig cfg = (BrickletLEDStripConfig) config.as(BrickletLEDStripConfig.class);
        switch(channel) {
            case "BrickletLEDStripLEDValues":
                if (command instanceof StringType) {
                    StringType cmd = (StringType) command;
if(Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(Helper.parseLED1ValueIndex(cmd.toString(), false, logger), Helper.parseLED1ValueLength(cmd.toString(), false, logger), Helper.parseLED1Values(cmd.toString(), 0, false, logger), Helper.parseLED1Values(cmd.toString(), 1, false, logger), Helper.parseLED1Values(cmd.toString(), 2, false, logger));
}
                }
        
if (command instanceof StringType) {
                    StringType cmd = (StringType) command;
if(!Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(Helper.parseLED1ValueIndex(cmd.toString(), true, logger), Helper.parseLED1ValueLength(cmd.toString(), true, logger), Helper.parseLED1Values(cmd.toString(), 0, true, logger), Helper.parseLED1Values(cmd.toString(), 1, true, logger), Helper.parseLED1Values(cmd.toString(), 2, true, logger), Helper.parseLED1Values(cmd.toString(), 3, true, logger));
}
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType, StringType.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletLEDStripAllLEDs":
                if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 0 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(0, (short)(channelCfg.ledCount >= 0 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 16 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(16, (short)(channelCfg.ledCount >= 16 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 32 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(32, (short)(channelCfg.ledCount >= 32 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 48 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(48, (short)(channelCfg.ledCount >= 48 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 64 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(64, (short)(channelCfg.ledCount >= 64 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 80 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(80, (short)(channelCfg.ledCount >= 80 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 96 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(96, (short)(channelCfg.ledCount >= 96 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 112 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(112, (short)(channelCfg.ledCount >= 112 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 128 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(128, (short)(channelCfg.ledCount >= 128 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 144 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(144, (short)(channelCfg.ledCount >= 144 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 160 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(160, (short)(channelCfg.ledCount >= 160 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 176 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(176, (short)(channelCfg.ledCount >= 176 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 192 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(192, (short)(channelCfg.ledCount >= 192 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 208 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(208, (short)(channelCfg.ledCount >= 208 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 224 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(224, (short)(channelCfg.ledCount >= 224 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 240 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(240, (short)(channelCfg.ledCount >= 240 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 256 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(256, (short)(channelCfg.ledCount >= 256 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 272 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(272, (short)(channelCfg.ledCount >= 272 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 288 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(288, (short)(channelCfg.ledCount >= 288 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 304 && Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBValues(304, (short)(channelCfg.ledCount >= 304 + 16 ? 16 : (channelCfg.ledCount % 16)), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, false, channelCfg.ledCount, 2));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 0 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(0, (short)(channelCfg.ledCount >= 0 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 12 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(12, (short)(channelCfg.ledCount >= 12 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 24 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(24, (short)(channelCfg.ledCount >= 24 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 36 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(36, (short)(channelCfg.ledCount >= 36 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 48 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(48, (short)(channelCfg.ledCount >= 48 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 60 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(60, (short)(channelCfg.ledCount >= 60 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 72 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(72, (short)(channelCfg.ledCount >= 72 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 84 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(84, (short)(channelCfg.ledCount >= 84 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 96 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(96, (short)(channelCfg.ledCount >= 96 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 108 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(108, (short)(channelCfg.ledCount >= 108 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 120 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(120, (short)(channelCfg.ledCount >= 120 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 132 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(132, (short)(channelCfg.ledCount >= 132 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 144 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(144, (short)(channelCfg.ledCount >= 144 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 156 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(156, (short)(channelCfg.ledCount >= 156 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 168 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(168, (short)(channelCfg.ledCount >= 168 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 180 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(180, (short)(channelCfg.ledCount >= 180 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 192 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(192, (short)(channelCfg.ledCount >= 192 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 204 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(204, (short)(channelCfg.ledCount >= 204 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 216 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(216, (short)(channelCfg.ledCount >= 216 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
if (command instanceof HSBType) {
                    BrickletLEDStripAllLEDsConfig channelCfg = channelConfig.as(BrickletLEDStripAllLEDsConfig.class);
HSBType cmd = (HSBType) command;
if(channelCfg.ledCount > 228 && !Arrays.asList(6, 9, 33, 36, 18, 24).contains(cfg.channelMapping)) {
    this.setRGBWValues(228, (short)(channelCfg.ledCount >= 228 + 12 ? 12 : (channelCfg.ledCount % 12)), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 0), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 1), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 2), Helper.createLED1ColorComponentList(cmd, true, channelCfg.ledCount, 3));
}
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType, HSBType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletLEDStripFrameStartedCallback0(int length, BrickletLEDStripConfig cfg) {
        return "";
    }

}