/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OnOffType;

import com.tinkerforge.BrickletLaserRangeFinder;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Laser Range Finder Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletLaserRangeFinderWrapper extends BrickletLaserRangeFinder implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletLaserRangeFinder.DEVICE_DISPLAY_NAME, "brickletlaserrangefinder", BrickletLaserRangeFinder.DEVICE_IDENTIFIER, BrickletLaserRangeFinderWrapper.class, BrickletLaserRangeFinderActions.class, "2.0.3", false);

    public BrickletLaserRangeFinderWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletLaserRangeFinderWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletLaserRangeFinderConfig cfg = (BrickletLaserRangeFinderConfig) config.as(BrickletLaserRangeFinderConfig.class);
        if(this.getSensorHardwareVersion() == 3 || cfg.mode == 0) {
this.addDistanceListener(this.reg((distance) -> {if(true) {updateStateFn.accept("BrickletLaserRangeFinderDistance", transformBrickletLaserRangeFinderDistanceCallback0(distance, cfg));}}, this::removeDistanceListener));}
		if(this.getSensorHardwareVersion() == 3 || cfg.mode != 0) {
this.addVelocityListener(this.reg((velocity) -> {if(true) {updateStateFn.accept("BrickletLaserRangeFinderVelocity", transformBrickletLaserRangeFinderVelocityCallback0(velocity, cfg));}}, this::removeVelocityListener));}
        if(this.getSensorHardwareVersion() == 1) {{
		        this.setMode(cfg.mode.shortValue());
		    }} else {{
		        this.setConfiguration(cfg.acquisitionCount.shortValue(), cfg.enableQuickTermination, cfg.thresholdValue.shortValue(), cfg.enableFixedMeasurementFrequency ? cfg.measurementFrequency : 0);
		    }}
		    this.setMovingAverage(cfg.distanceMovingAverageLength.shortValue(), cfg.velocityMovingAverageLength.shortValue());
		if (this.getSensorHardwareVersion() == 3 || cfg.mode == 0) {
		BrickletLaserRangeFinderDistanceConfig channelCfg = getChannelConfigFn.apply("BrickletLaserRangeFinderDistance").as(BrickletLaserRangeFinderDistanceConfig.class);
		this.setDistanceCallbackPeriod(channelCfg.updateInterval);
		this.setDistanceCallbackThreshold('x', 0, 0);
		}
		if (this.getSensorHardwareVersion() == 3 || cfg.mode != 0) {
		BrickletLaserRangeFinderVelocityConfig channelCfg = getChannelConfigFn.apply("BrickletLaserRangeFinderVelocity").as(BrickletLaserRangeFinderVelocityConfig.class);
		this.setVelocityCallbackPeriod(channelCfg.updateInterval);
		this.setVelocityCallbackThreshold('x', (short)0, (short)0);
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        BrickletLaserRangeFinderConfig cfg = (BrickletLaserRangeFinderConfig) config.as(BrickletLaserRangeFinderConfig.class);
        if(this.getSensorHardwareVersion() == 3 || cfg.mode == 0) {
this.setDistanceCallbackPeriod(0);}
		if(this.getSensorHardwareVersion() == 3 || cfg.mode != 0) {
this.setVelocityCallbackPeriod(0);}
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        BrickletLaserRangeFinderConfig cfg = (BrickletLaserRangeFinderConfig) config.as(BrickletLaserRangeFinderConfig.class);
List<String> result = new ArrayList<String>();
        if (this.getSensorHardwareVersion() == 3 || cfg.mode == 0) {
                result.add("BrickletLaserRangeFinderDistance");
            }
		if (this.getSensorHardwareVersion() == 3 || cfg.mode != 0) {
                result.add("BrickletLaserRangeFinderVelocity");
            }
		result.add("BrickletLaserRangeFinderLaser");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletLaserRangeFinderDistance":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderDistance"), "Distance", "Number:Length").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderDistance")).withDescription("The measured distance. Sensor hardware version 1 cannot measure distance and velocity at the same time. Therefore, the distance mode has to be enabled. Sensor hardware version 3 can measure distance and velocity at the same time. In both cases the laser has to be enabled.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(40)).withStep(BigDecimal.valueOf(0.01)).withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletLaserRangeFinderVelocity":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderVelocity"), "Velocity", "Number:Speed").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderVelocity")).withDescription("The measured velocity. Sensor hardware version 1 cannot measure distance and velocity at the same time. Therefore, the velocity mode has to be enabled. Sensor hardware version 3 can measure distance and velocity at the same time, but the velocity measurement only produces stables results if a fixed measurement frequency is configured. Also the laser has to be enabled.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(-128)).withMaximum(BigDecimal.valueOf(127)).withStep(BigDecimal.valueOf(0.01)).withPattern("%.2f %unit%").withReadOnly(true).build().toStateDescription()).build();
            case "BrickletLaserRangeFinderLaser":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderLaser"), "Laser", "Switch").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletLaserRangeFinderLaser")).withDescription("Activates/Deactivates the laser of the LIDAR.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Laser Range Finder Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Measures distance up to 40m with laser light").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletLaserRangeFinderDistance", new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderDistance")).withLabel("Distance").build(), new ChannelDefinitionBuilder("BrickletLaserRangeFinderVelocity", new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderVelocity")).withLabel("Velocity").build(), new ChannelDefinitionBuilder("BrickletLaserRangeFinderLaser", new ChannelTypeUID("tinkerforge", "BrickletLaserRangeFinderLaser")).withLabel("Laser").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletlaserrangefinder":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("distanceMovingAverageLength", Type.INTEGER).withDefault("10").withDescription("The length of a moving averaging for the distance.").withGroupName("average").withLabel("Distance Moving Average Length").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(30)).build(), ConfigDescriptionParameterBuilder.create("velocityMovingAverageLength", Type.INTEGER).withDefault("10").withDescription("The length of a moving averaging for the velocity.").withGroupName("average").withLabel("Velocity Moving Average Length").withAdvanced(true).withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(30)).build(), ConfigDescriptionParameterBuilder.create("mode", Type.INTEGER).withDefault("0").withDescription("The LIDAR-Lite sensor (hardware version 1) has five different modes. One mode is for distance measurements and four modes are for velocity measurements with different ranges.<br/><br/>The following modes are available:<br/><ul><li>0: Distance is measured with resolution 1.0 cm and range 0-400 cm</li><li>1: Velocity is measured with resolution 0.1 m/s and range is 0-12.7 m/s</li><li>2: Velocity is measured with resolution 0.25 m/s and range is 0-31.75 m/s</li><li>3: Velocity is measured with resolution 0.5 m/s and range is 0-63.5 m/s</li><li>4: Velocity is measured with resolution 1.0 m/s and range is 0-127 m/s</li></ul>This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 3.").withGroupName("sensor1").withLabel("Mode").withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Distance"), new ParameterOption("1", "Velocity Max 13m/s"), new ParameterOption("2", "Velocity Max 32m/s"), new ParameterOption("3", "Velocity Max 64m/s"), new ParameterOption("4", "Velocity Max 127m/s"))).build(), ConfigDescriptionParameterBuilder.create("acquisitionCount", Type.INTEGER).withDefault("128").withDescription("The acquisition count defines the number of times the Laser Range Finder Bricklet will integrate acquisitions to find a correlation record peak. With a higher count, the Bricklet can measure longer distances. With a lower count, the rate increases. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.").withGroupName("sensor3").withLabel("Acquisition Count").withMinimum(BigDecimal.valueOf(1)).withMaximum(BigDecimal.valueOf(255)).build(), ConfigDescriptionParameterBuilder.create("enableQuickTermination", Type.BOOLEAN).withDefault("false").withDescription("If you enable Quick Termination, the distance measurement will be terminated early if a high peak was already detected. This means that a higher measurement rate can be achieved and long distances can be measured at the same time. However, the chance of false-positive distance measurements increases. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.").withGroupName("sensor3").withLabel("Quick Termination").build(), ConfigDescriptionParameterBuilder.create("thresholdValue", Type.INTEGER).withDefault("0").withDescription("Normally the distance is calculated with a detection algorithm that uses peak value, signal strength and noise. You can however also define a fixed Threshold Value. Set this to a low value if you want to measure the distance to something that has very little reflection (e.g. glass) and set it to a high value if you want to measure the distance to something with a very high reflection (e.g. mirror). Set this to 0 to use the default algorithm. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.").withGroupName("sensor3").withLabel("Threshold Value").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(255)).build(), ConfigDescriptionParameterBuilder.create("enableFixedMeasurementFrequency", Type.BOOLEAN).withDefault("false").withDescription("For a stable velocity measurement you should set a fixed measurement frequency. See Measurement Frequency for details. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.").withGroupName("sensor3").withLabel("Fixed Measurement Frequency").build(), ConfigDescriptionParameterBuilder.create("measurementFrequency", Type.INTEGER).withDefault("10").withDescription("If the fixed measurement frequency is enabled, the measurement frequency is forced to this value. If it is disabled, the Laser Range Finder Bricklet will use the optimal frequency according to the other configurations and the actual measured distance. Since the rate is not fixed in this case, the velocity measurement is not stable. For a stable velocity measurement you should set a fixed measurement frequency. The lower the frequency, the higher is the resolution of the calculated velocity. This setting will be ignored if you have a LIDAR-Lite sensor with hardware version 1.").withGroupName("sensor3").withLabel("Measurement Frequency").withMinimum(BigDecimal.valueOf(10)).withMaximum(BigDecimal.valueOf(500)).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"), new ConfigDescriptionParameterGroup("average", "null", true, "Averaging", "The averaging parameters."), new ConfigDescriptionParameterGroup("sensor1", "null", false, "Sensor Version 1", "Configuration for LIDAR-Lite sensors with hardware version 1"), new ConfigDescriptionParameterGroup("sensor3", "null", false, "Sensor Version 3", "Configuration for LIDAR-Lite sensors with hardware version 3"))).build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderDistance":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderVelocity":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletLaserRangeFinderLaser":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletLaserRangeFinderConfig cfg = (BrickletLaserRangeFinderConfig) config.as(BrickletLaserRangeFinderConfig.class);
switch(channel) {
            case "BrickletLaserRangeFinderDistance":
                   {
            int value = this.getDistance();
            if(true)
                updateStateFn.accept(channel, transformBrickletLaserRangeFinderDistanceGetter0(value, cfg));
        }
                   return;
            case "BrickletLaserRangeFinderVelocity":
                   {
            short value = this.getVelocity();
            if(true)
                updateStateFn.accept(channel, transformBrickletLaserRangeFinderVelocityGetter0(value, cfg));
        }
                   return;
            case "BrickletLaserRangeFinderLaser":
                   {
            boolean value = this.isLaserEnabled();
            if(true)
                updateStateFn.accept(channel, transformBrickletLaserRangeFinderLaserGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletLaserRangeFinderLaser":
                if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
if(cmd == OnOffType.ON) {
    this.enableLaser();
}
                }
        
if (command instanceof OnOffType) {
                    OnOffType cmd = (OnOffType) command;
if(cmd == OnOffType.OFF) {
    this.disableLaser();
}
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of OnOffType, OnOffType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderDistanceCallback0(int distance, BrickletLaserRangeFinderConfig cfg) {
        return new QuantityType<>(distance / 100.0, SIUnits.METRE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderVelocityCallback0(short velocity, BrickletLaserRangeFinderConfig cfg) {
        return new QuantityType<>(velocity / 100.0, SmartHomeUnits.METRE_PER_SECOND);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderDistanceGetter0(int value, BrickletLaserRangeFinderConfig cfg) {
        return new QuantityType<>(value / 100.0, SIUnits.METRE);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderVelocityGetter0(short value, BrickletLaserRangeFinderConfig cfg) {
        return new QuantityType<>(value / 100.0, SmartHomeUnits.METRE_PER_SECOND);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletLaserRangeFinderLaserGetter0(boolean value, BrickletLaserRangeFinderConfig cfg) {
        return value? OnOffType.ON : OnOffType.OFF;
    }

}