/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletLoadCellV2;

/**
 * Passes through actions of the Load Cell Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLoadCellV2Actions implements ThingActions, IBrickletLoadCellV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletLoadCellV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletLoadCellV2Actions.class.getName())) {
            if (actions instanceof IBrickletLoadCellV2Actions) {
                return (IBrickletLoadCellV2Actions) actions;
            } else {
                return (IBrickletLoadCellV2Actions) Proxy.newProxyInstance(IBrickletLoadCellV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletLoadCellV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletLoadCellV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Weight")
    public @ActionOutput(name = "weight", type="int")
           Map<String, Object> brickletLoadCellV2GetWeight(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getWeight();
        
        result.put("weight", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetWeight(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetWeight();
    }


    @Override
    @RuleAction(label = "Get Info LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletLoadCellV2GetInfoLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getInfoLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetInfoLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetInfoLEDConfig();
    }

    @Override
    @RuleAction(label = "Tare")
    public void brickletLoadCellV2Tare(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.tare();
        
    }

    public static void brickletLoadCellV2Tare(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLoadCellV2Tare();
    }


    @Override
    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="int")
           Map<String, Object> brickletLoadCellV2GetMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getMovingAverage();
        
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetMovingAverage();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "rate", type="int")
           @ActionOutput(name = "gain", type="int")
           Map<String, Object> brickletLoadCellV2GetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLoadCellV2.Configuration value = dev.getConfiguration();
        
        result.put("rate", value.rate);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetConfiguration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletLoadCellV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletLoadCellV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletLoadCellV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLoadCellV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletLoadCellV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletLoadCellV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletLoadCellV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletLoadCellV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletLoadCellV2Wrapper dev = ((@Nullable BrickletLoadCellV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletLoadCellV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletLoadCellV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletLoadCellV2GetIdentity();
    }
}
