/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletMotionDetectorV2;

/**
 * Passes through actions of the Motion Detector Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotionDetectorV2Actions implements ThingActions, IBrickletMotionDetectorV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletMotionDetectorV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletMotionDetectorV2Actions.class.getName())) {
            if (actions instanceof IBrickletMotionDetectorV2Actions) {
                return (IBrickletMotionDetectorV2Actions) actions;
            } else {
                return (IBrickletMotionDetectorV2Actions) Proxy.newProxyInstance(IBrickletMotionDetectorV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletMotionDetectorV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletMotionDetectorV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Motion Detected")
    public @ActionOutput(name = "motion", type="int")
           Map<String, Object> brickletMotionDetectorV2GetMotionDetected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getMotionDetected();
        
        result.put("motion", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetMotionDetected(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetMotionDetected();
    }


    @Override
    @RuleAction(label = "Get Sensitivity")
    public @ActionOutput(name = "sensitivity", type="int")
           Map<String, Object> brickletMotionDetectorV2GetSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getSensitivity();
        
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetSensitivity();
    }

    @Override
    @RuleAction(label = "Set Indicator")
    public void brickletMotionDetectorV2SetIndicator(
            @ActionInput(name = "topLeft") int topLeft,
            @ActionInput(name = "topRight") int topRight,
            @ActionInput(name = "bottom") int bottom) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setIndicator(topLeft, topRight, bottom);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopLeftIndicator")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopLeftIndicator"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopRightIndicator")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2TopRightIndicator"), RefreshType.REFRESH);
}
if(t.getChannel(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2BottomIndicator")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletMotionDetectorV2BottomIndicator"), RefreshType.REFRESH);
}
    }

    public static void brickletMotionDetectorV2SetIndicator(@Nullable ThingActions actions, int topLeft, int topRight, int bottom) throws TinkerforgeException {
        invokeMethodOf(actions).brickletMotionDetectorV2SetIndicator(topLeft, topRight, bottom);
    }


    @Override
    @RuleAction(label = "Get Indicator")
    public @ActionOutput(name = "topLeft", type="int")
           @ActionOutput(name = "topRight", type="int")
           @ActionOutput(name = "bottom", type="int")
           Map<String, Object> brickletMotionDetectorV2GetIndicator(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.Indicator value = dev.getIndicator();
        
        result.put("topLeft", value.topLeft);
        result.put("topRight", value.topRight);
        result.put("bottom", value.bottom);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIndicator(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetIndicator();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletMotionDetectorV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletMotionDetectorV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletMotionDetectorV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletMotionDetectorV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletMotionDetectorV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletMotionDetectorV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletMotionDetectorV2Wrapper dev = ((@Nullable BrickletMotionDetectorV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletMotionDetectorV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletMotionDetectorV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletMotionDetectorV2GetIdentity();
    }
}
