/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;

import com.tinkerforge.BrickletMotionDetectorV2;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Motion Detector Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletMotionDetectorV2Wrapper extends BrickletMotionDetectorV2 implements CoMCUFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletMotionDetectorV2.DEVICE_DISPLAY_NAME, "brickletmotiondetectorv2", BrickletMotionDetectorV2.DEVICE_IDENTIFIER, BrickletMotionDetectorV2Wrapper.class, BrickletMotionDetectorV2Actions.class, "2.0.0", true);

    public BrickletMotionDetectorV2Wrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletMotionDetectorV2Wrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletMotionDetectorV2Config cfg = (BrickletMotionDetectorV2Config) config.as(BrickletMotionDetectorV2Config.class);
        this.addMotionDetectedListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletMotionDetectorV2MotionDetected", transformBrickletMotionDetectorV2MotionDetectedCallback0(cfg));}}, this::removeMotionDetectedListener));
		this.addDetectionCycleEndedListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletMotionDetectorV2DetectionCycleEnded", transformBrickletMotionDetectorV2DetectionCycleEndedCallback0(cfg));}}, this::removeDetectionCycleEndedListener));
        this.setSensitivity(cfg.sensitivity);
		this.setStatusLEDConfig(cfg.statusLEDConfig);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletMotionDetectorV2MotionDetected");
		result.add("BrickletMotionDetectorV2DetectionCycleEnded");
		result.add("BrickletMotionDetectorV2TopLeftIndicator");
		result.add("BrickletMotionDetectorV2TopRightIndicator");
		result.add("BrickletMotionDetectorV2BottomIndicator");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletMotionDetectorV2Indicator":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletMotionDetectorV2Indicator"), "NOT USED", "Number:Dimensionless").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletMotionDetectorV2Indicator")).withDescription("One of the blue backlight LEDs of the fresnel lens. A value of 0 turns the LED off and a value of 255 turns the LED to full brightness.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(255)).withPattern("%d").withReadOnly(false).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Motion Detector Bricklet 2.0").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Passive infrared (PIR) motion sensor with 12m range and dimmable backlight").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletMotionDetectorV2MotionDetected", new ChannelTypeUID("system", "trigger")).withLabel("Motion Detected").withDescription("This channel is triggered after a motion was detected.").build(), new ChannelDefinitionBuilder("BrickletMotionDetectorV2DetectionCycleEnded", new ChannelTypeUID("system", "trigger")).withLabel("Detection Cycle Ended").withDescription("This channel is triggered when the detection cycle ended. A new motion can be detected again after approximately 2 seconds.").build(), new ChannelDefinitionBuilder("BrickletMotionDetectorV2TopLeftIndicator", new ChannelTypeUID("tinkerforge", "BrickletMotionDetectorV2Indicator")).withLabel("Top Left Indicator").build(), new ChannelDefinitionBuilder("BrickletMotionDetectorV2TopRightIndicator", new ChannelTypeUID("tinkerforge", "BrickletMotionDetectorV2Indicator")).withLabel("Top Right Indicator").build(), new ChannelDefinitionBuilder("BrickletMotionDetectorV2BottomIndicator", new ChannelTypeUID("tinkerforge", "BrickletMotionDetectorV2Indicator")).withLabel("Bottom Indicator").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletmotiondetectorv2":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("sensitivity", Type.INTEGER).withDefault("50").withDescription("The sensitivity of the PIR sensor. The range is 0-100. At full sensitivity (100), the Bricklet can detect motion in a range of approximately 12m.<br/><br/>The actual range depends on many things in the environment (e.g. reflections) and the size of the object to be detected. While a big person might be detected in a range of 10m a cat may only be detected at 2m distance with the same setting. So you will have to find a good sensitivity for your application by trial and error.").withLabel("Sensitivity").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(100)).build(), ConfigDescriptionParameterBuilder.create("statusLEDConfig", Type.INTEGER).withDefault("3").withDescription("The status LED configuration. By default the LED shows communication traffic between Brick and Bricklet, it flickers once for every 10 received data packets.<br/><br/>You can also turn the LED permanently on/off or show a heartbeat.<br/><br/>If the Bricklet is in bootloader mode, the LED is will show heartbeat by default.").withLabel("Status LED Configuration").withAdvanced(true).withLimitToOptions(true).withOptions(Arrays.asList(new ParameterOption("0", "Off"), new ParameterOption("1", "On"), new ParameterOption("2", "Show Heartbeat"), new ParameterOption("3", "Show Status"))).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletMotionDetectorV2Indicator":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletMotionDetectorV2Config cfg = (BrickletMotionDetectorV2Config) config.as(BrickletMotionDetectorV2Config.class);
switch(channel) {
            case "BrickletMotionDetectorV2MotionDetected":
               return;
            case "BrickletMotionDetectorV2DetectionCycleEnded":
               return;
            case "BrickletMotionDetectorV2TopLeftIndicator":
                   {
            BrickletMotionDetectorV2.Indicator value = this.getIndicator();
            if(true)
                updateStateFn.accept(channel, transformBrickletMotionDetectorV2TopLeftIndicatorGetter0(value, cfg));
        }
                   return;
            case "BrickletMotionDetectorV2TopRightIndicator":
                   {
            BrickletMotionDetectorV2.Indicator value = this.getIndicator();
            if(true)
                updateStateFn.accept(channel, transformBrickletMotionDetectorV2TopRightIndicatorGetter0(value, cfg));
        }
                   return;
            case "BrickletMotionDetectorV2BottomIndicator":
                   {
            BrickletMotionDetectorV2.Indicator value = this.getIndicator();
            if(true)
                updateStateFn.accept(channel, transformBrickletMotionDetectorV2BottomIndicatorGetter0(value, cfg));
        }
                   return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletMotionDetectorV2TopLeftIndicator":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
this.setIndicator(cmd.intValue(), this.getIndicator().topRight, this.getIndicator().bottom);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletMotionDetectorV2TopRightIndicator":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
this.setIndicator(this.getIndicator().topLeft, cmd.intValue(), this.getIndicator().bottom);
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletMotionDetectorV2BottomIndicator":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
this.setIndicator(this.getIndicator().topLeft, this.getIndicator().topRight, cmd.intValue());
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletMotionDetectorV2MotionDetectedCallback0(BrickletMotionDetectorV2Config cfg) {
        return "";
    }
	    private String transformBrickletMotionDetectorV2DetectionCycleEndedCallback0(BrickletMotionDetectorV2Config cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletMotionDetectorV2TopLeftIndicatorGetter0(BrickletMotionDetectorV2.Indicator value, BrickletMotionDetectorV2Config cfg) {
        return new DecimalType(value.topLeft);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletMotionDetectorV2TopRightIndicatorGetter0(BrickletMotionDetectorV2.Indicator value, BrickletMotionDetectorV2Config cfg) {
        return new DecimalType(value.topRight);
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletMotionDetectorV2BottomIndicatorGetter0(BrickletMotionDetectorV2.Indicator value, BrickletMotionDetectorV2Config cfg) {
        return new DecimalType(value.bottom);
    }

}