/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletOneWire;

/**
 * Passes through actions of the One Wire Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOneWireActions implements ThingActions, IBrickletOneWireActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletOneWireActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletOneWireActions.class.getName())) {
            if (actions instanceof IBrickletOneWireActions) {
                return (IBrickletOneWireActions) actions;
            } else {
                return (IBrickletOneWireActions) Proxy.newProxyInstance(IBrickletOneWireActions.class.getClassLoader(),
                        new Class[] { IBrickletOneWireActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletOneWireActions");
    }

    
    @Override
    @RuleAction(label = "Search Bus")
    public @ActionOutput(name = "identifier", type="long[]")
           @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireSearchBus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOneWire.SearchBus value = dev.searchBus();
        
        result.put("identifier", value.identifier);
        result.put("status", value.status);
        return result;
    }

    public static Map<String, Object> brickletOneWireSearchBus(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireSearchBus();
    }


    @Override
    @RuleAction(label = "Reset Bus")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireResetBus(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.resetBus();
        
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireResetBus(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireResetBus();
    }


    @Override
    @RuleAction(label = "Write")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireWrite(
            @ActionInput(name = "data") int data) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.write(data);
        
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireWrite(@Nullable ThingActions actions, int data) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireWrite(data);
    }


    @Override
    @RuleAction(label = "Read")
    public @ActionOutput(name = "data", type="int")
           @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireRead(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOneWire.Read value = dev.read();
        
        result.put("data", value.data);
        result.put("status", value.status);
        return result;
    }

    public static Map<String, Object> brickletOneWireRead(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireRead();
    }


    @Override
    @RuleAction(label = "Write Command")
    public @ActionOutput(name = "status", type="int")
           Map<String, Object> brickletOneWireWriteCommand(
            @ActionInput(name = "identifier") long identifier,
            @ActionInput(name = "command") int command) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.writeCommand(identifier, command);
        
        result.put("status", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireWriteCommand(@Nullable ThingActions actions, long identifier, int command) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireWriteCommand(identifier, command);
    }


    @Override
    @RuleAction(label = "Get Communication LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOneWireGetCommunicationLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getCommunicationLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetCommunicationLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireGetCommunicationLEDConfig();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletOneWireReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletOneWireGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletOneWireGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletOneWireGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOneWire.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletOneWireReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletOneWireReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletOneWireReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletOneWireGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletOneWireWrapper dev = ((@Nullable BrickletOneWireWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletOneWire.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletOneWireGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletOneWireGetIdentity();
    }
}
