/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletPTCV2;

/**
 * Passes through actions of the PTC Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletPTCV2Actions implements ThingActions, IBrickletPTCV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletPTCV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletPTCV2Actions.class.getName())) {
            if (actions instanceof IBrickletPTCV2Actions) {
                return (IBrickletPTCV2Actions) actions;
            } else {
                return (IBrickletPTCV2Actions) Proxy.newProxyInstance(IBrickletPTCV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletPTCV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletPTCV2Actions");
    }

    
    @Override
    @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPTCV2GetTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetTemperature();
    }


    @Override
    @RuleAction(label = "Is Sensor Connected")
    public @ActionOutput(name = "connected", type="boolean")
           Map<String, Object> brickletPTCV2IsSensorConnected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        boolean value = dev.isSensorConnected();
        
        result.put("connected", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2IsSensorConnected(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2IsSensorConnected();
    }


    @Override
    @RuleAction(label = "Get Wire Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletPTCV2GetWireMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getWireMode();
        
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetWireMode(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetWireMode();
    }


    @Override
    @RuleAction(label = "Get Resistance")
    public @ActionOutput(name = "resistance", type="int")
           Map<String, Object> brickletPTCV2GetResistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getResistance();
        
        result.put("resistance", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetResistance(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetResistance();
    }


    @Override
    @RuleAction(label = "Get Noise Rejection Filter")
    public @ActionOutput(name = "filter", type="int")
           Map<String, Object> brickletPTCV2GetNoiseRejectionFilter(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getNoiseRejectionFilter();
        
        result.put("filter", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetNoiseRejectionFilter(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetNoiseRejectionFilter();
    }


    @Override
    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLengthResistance", type="int")
           @ActionOutput(name = "movingAverageLengthTemperature", type="int")
           Map<String, Object> brickletPTCV2GetMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletPTCV2.MovingAverageConfiguration value = dev.getMovingAverageConfiguration();
        
        result.put("movingAverageLengthResistance", value.movingAverageLengthResistance);
        result.put("movingAverageLengthTemperature", value.movingAverageLengthTemperature);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetMovingAverageConfiguration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletPTCV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletPTCV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletPTCV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletPTCV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletPTCV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletPTCV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletPTCV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletPTCV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletPTCV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletPTCV2Wrapper dev = ((@Nullable BrickletPTCV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletPTCV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletPTCV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletPTCV2GetIdentity();
    }
}
