/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.StringType;

import com.tinkerforge.BrickletPiezoBuzzer;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Piezo Buzzer Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletPiezoBuzzerWrapper extends BrickletPiezoBuzzer implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletPiezoBuzzer.DEVICE_DISPLAY_NAME, "brickletpiezobuzzer", BrickletPiezoBuzzer.DEVICE_IDENTIFIER, BrickletPiezoBuzzerWrapper.class, BrickletPiezoBuzzerActions.class, "2.0.0", false);

    public BrickletPiezoBuzzerWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletPiezoBuzzerWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletPiezoBuzzerConfig cfg = (BrickletPiezoBuzzerConfig) config.as(BrickletPiezoBuzzerConfig.class);
        this.addBeepFinishedListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletPiezoBuzzerBeepFinished", transformBrickletPiezoBuzzerBeepFinishedCallback0(cfg));}}, this::removeBeepFinishedListener));
		this.addMorseCodeFinishedListener(this.reg(() -> {if(true) {triggerChannelFn.accept("BrickletPiezoBuzzerMorseCodeFinished", transformBrickletPiezoBuzzerMorseCodeFinishedCallback0(cfg));}}, this::removeMorseCodeFinishedListener));
        
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletPiezoBuzzerBeep");
		result.add("BrickletPiezoBuzzerMorseCode");
		result.add("BrickletPiezoBuzzerBeepFinished");
		result.add("BrickletPiezoBuzzerMorseCodeFinished");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletPiezoBuzzerBeep":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletPiezoBuzzerBeep"), "Beep", "Number:Time").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletPiezoBuzzerBeep")).withDescription("Beeps with the duration in s.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(false).build().toStateDescription()).build();
            case "BrickletPiezoBuzzerMorseCode":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletPiezoBuzzerMorseCode"), "Morse Code", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletPiezoBuzzerMorseCode")).withDescription("Morse code that will be played by the piezo buzzer. The morse code is given as a string consisting of '.' (dot), '-' (minus) and ' ' (space) for dits, dahs and pauses. Every other character is ignored. For example: If you set the string '...---...', the piezo buzzer will beep nine times with the durations 'short short short long long long short short short'. The maximum string size is 60.").build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Piezo Buzzer Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Creates 1kHz beep").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletPiezoBuzzerBeep", new ChannelTypeUID("tinkerforge", "BrickletPiezoBuzzerBeep")).withLabel("Beep").build(), new ChannelDefinitionBuilder("BrickletPiezoBuzzerMorseCode", new ChannelTypeUID("tinkerforge", "BrickletPiezoBuzzerMorseCode")).withLabel("Morse Code").build(), new ChannelDefinitionBuilder("BrickletPiezoBuzzerBeepFinished", new ChannelTypeUID("system", "trigger")).withLabel("Beep Finished").withDescription("This channel is triggered if a beep set by the beep action is finished.").build(), new ChannelDefinitionBuilder("BrickletPiezoBuzzerMorseCodeFinished", new ChannelTypeUID("system", "trigger")).withLabel("Morse Code Finished").withDescription("This channel is triggered if the playback of the morse code set by the morseCode action is finished.").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletpiezobuzzer":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletPiezoBuzzerBeep":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletPiezoBuzzerMorseCode":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        switch(channel) {
            case "BrickletPiezoBuzzerBeep":
               return;
            case "BrickletPiezoBuzzerMorseCode":
               return;
            case "BrickletPiezoBuzzerBeepFinished":
               return;
            case "BrickletPiezoBuzzerMorseCodeFinished":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletPiezoBuzzerBeep":
                if (command instanceof Number) {
                    Number cmd = (Number) command;
this.beep((long)(cmd.doubleValue() * 1000.0));
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of Number.", command.getClass().getName(), channel);
                }
                
                break;
            case "BrickletPiezoBuzzerMorseCode":
                if (command instanceof StringType) {
                    StringType cmd = (StringType) command;
this.morseCode(cmd.toString());
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of StringType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private String transformBrickletPiezoBuzzerBeepFinishedCallback0(BrickletPiezoBuzzerConfig cfg) {
        return "";
    }
	    private String transformBrickletPiezoBuzzerMorseCodeFinishedCallback0(BrickletPiezoBuzzerConfig cfg) {
        return "";
    }

}