/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRealTimeClock;

/**
 * Passes through actions of the Real-Time Clock Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRealTimeClockActions implements ThingActions, IBrickletRealTimeClockActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletRealTimeClockActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletRealTimeClockActions.class.getName())) {
            if (actions instanceof IBrickletRealTimeClockActions) {
                return (IBrickletRealTimeClockActions) actions;
            } else {
                return (IBrickletRealTimeClockActions) Proxy.newProxyInstance(IBrickletRealTimeClockActions.class.getClassLoader(),
                        new Class[] { IBrickletRealTimeClockActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletRealTimeClockActions");
    }

        @Override
    @RuleAction(label = "Set Date Time")
    public void brickletRealTimeClockSetDateTime(
            @ActionInput(name = "year") int year,
            @ActionInput(name = "month") short month,
            @ActionInput(name = "day") short day,
            @ActionInput(name = "hour") short hour,
            @ActionInput(name = "minute") short minute,
            @ActionInput(name = "second") short second,
            @ActionInput(name = "centisecond") short centisecond,
            @ActionInput(name = "weekday") short weekday) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setDateTime(year, month, day, hour, minute, second, centisecond, weekday);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletRealTimeClockDateTime")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletRealTimeClockDateTime"), RefreshType.REFRESH);
}
    }

    public static void brickletRealTimeClockSetDateTime(@Nullable ThingActions actions, int year, short month, short day, short hour, short minute, short second, short centisecond, short weekday) throws TinkerforgeException {
        invokeMethodOf(actions).brickletRealTimeClockSetDateTime(year, month, day, hour, minute, second, centisecond, weekday);
    }


    @Override
    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "year", type="int")
           @ActionOutput(name = "month", type="short")
           @ActionOutput(name = "day", type="short")
           @ActionOutput(name = "hour", type="short")
           @ActionOutput(name = "minute", type="short")
           @ActionOutput(name = "second", type="short")
           @ActionOutput(name = "centisecond", type="short")
           @ActionOutput(name = "weekday", type="short")
           Map<String, Object> brickletRealTimeClockGetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClock.DateTime value = dev.getDateTime();
        
        result.put("year", value.year);
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("centisecond", value.centisecond);
        result.put("weekday", value.weekday);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockGetDateTime();
    }


    @Override
    @RuleAction(label = "Get Timestamp")
    public @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockGetTimestamp(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getTimestamp();
        
        result.put("timestamp", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetTimestamp(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockGetTimestamp();
    }


    @Override
    @RuleAction(label = "Get Offset")
    public @ActionOutput(name = "offset", type="byte")
           Map<String, Object> brickletRealTimeClockGetOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        byte value = dev.getOffset();
        
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockGetOffset();
    }

    @Override
    @RuleAction(label = "Set Alarm")
    public void brickletRealTimeClockSetAlarm(
            @ActionInput(name = "month") byte month,
            @ActionInput(name = "day") byte day,
            @ActionInput(name = "hour") byte hour,
            @ActionInput(name = "minute") byte minute,
            @ActionInput(name = "second") byte second,
            @ActionInput(name = "weekday") byte weekday,
            @ActionInput(name = "interval") int interval) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setAlarm(month, day, hour, minute, second, weekday, interval);
        
    }

    public static void brickletRealTimeClockSetAlarm(@Nullable ThingActions actions, byte month, byte day, byte hour, byte minute, byte second, byte weekday, int interval) throws TinkerforgeException {
        invokeMethodOf(actions).brickletRealTimeClockSetAlarm(month, day, hour, minute, second, weekday, interval);
    }


    @Override
    @RuleAction(label = "Get Alarm")
    public @ActionOutput(name = "month", type="byte")
           @ActionOutput(name = "day", type="byte")
           @ActionOutput(name = "hour", type="byte")
           @ActionOutput(name = "minute", type="byte")
           @ActionOutput(name = "second", type="byte")
           @ActionOutput(name = "weekday", type="byte")
           @ActionOutput(name = "interval", type="int")
           Map<String, Object> brickletRealTimeClockGetAlarm(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClock.Alarm value = dev.getAlarm();
        
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("weekday", value.weekday);
        result.put("interval", value.interval);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetAlarm(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockGetAlarm();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRealTimeClockGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockWrapper dev = ((@Nullable BrickletRealTimeClockWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClock.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockGetIdentity();
    }
}
