/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletRealTimeClockV2;

/**
 * Passes through actions of the Real-Time Clock Bricklet 2.0.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRealTimeClockV2Actions implements ThingActions, IBrickletRealTimeClockV2Actions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletRealTimeClockV2Actions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletRealTimeClockV2Actions.class.getName())) {
            if (actions instanceof IBrickletRealTimeClockV2Actions) {
                return (IBrickletRealTimeClockV2Actions) actions;
            } else {
                return (IBrickletRealTimeClockV2Actions) Proxy.newProxyInstance(IBrickletRealTimeClockV2Actions.class.getClassLoader(),
                        new Class[] { IBrickletRealTimeClockV2Actions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletRealTimeClockV2Actions");
    }

        @Override
    @RuleAction(label = "Set Date Time")
    public void brickletRealTimeClockV2SetDateTime(
            @ActionInput(name = "year") int year,
            @ActionInput(name = "month") int month,
            @ActionInput(name = "day") int day,
            @ActionInput(name = "hour") int hour,
            @ActionInput(name = "minute") int minute,
            @ActionInput(name = "second") int second,
            @ActionInput(name = "centisecond") int centisecond,
            @ActionInput(name = "weekday") int weekday) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        Thing t = h.getThing();
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setDateTime(year, month, day, hour, minute, second, centisecond, weekday);
        if(t.getChannel(new ChannelUID(t.getUID(), "BrickletRealTimeClockV2DateTime")) != null){
	h.handleCommand(new ChannelUID(t.getUID(), "BrickletRealTimeClockV2DateTime"), RefreshType.REFRESH);
}
    }

    public static void brickletRealTimeClockV2SetDateTime(@Nullable ThingActions actions, int year, int month, int day, int hour, int minute, int second, int centisecond, int weekday) throws TinkerforgeException {
        invokeMethodOf(actions).brickletRealTimeClockV2SetDateTime(year, month, day, hour, minute, second, centisecond, weekday);
    }


    @Override
    @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "year", type="int")
           @ActionOutput(name = "month", type="int")
           @ActionOutput(name = "day", type="int")
           @ActionOutput(name = "hour", type="int")
           @ActionOutput(name = "minute", type="int")
           @ActionOutput(name = "second", type="int")
           @ActionOutput(name = "centisecond", type="int")
           @ActionOutput(name = "weekday", type="int")
           @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockV2GetDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClockV2.DateTime value = dev.getDateTime();
        
        result.put("year", value.year);
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("centisecond", value.centisecond);
        result.put("weekday", value.weekday);
        result.put("timestamp", value.timestamp);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetDateTime();
    }


    @Override
    @RuleAction(label = "Get Timestamp")
    public @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> brickletRealTimeClockV2GetTimestamp(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.getTimestamp();
        
        result.put("timestamp", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetTimestamp(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetTimestamp();
    }


    @Override
    @RuleAction(label = "Get Offset")
    public @ActionOutput(name = "offset", type="int")
           Map<String, Object> brickletRealTimeClockV2GetOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getOffset();
        
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetOffset();
    }

    @Override
    @RuleAction(label = "Set Alarm")
    public void brickletRealTimeClockV2SetAlarm(
            @ActionInput(name = "month") int month,
            @ActionInput(name = "day") int day,
            @ActionInput(name = "hour") int hour,
            @ActionInput(name = "minute") int minute,
            @ActionInput(name = "second") int second,
            @ActionInput(name = "weekday") int weekday,
            @ActionInput(name = "interval") int interval) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.setAlarm(month, day, hour, minute, second, weekday, interval);
        
    }

    public static void brickletRealTimeClockV2SetAlarm(@Nullable ThingActions actions, int month, int day, int hour, int minute, int second, int weekday, int interval) throws TinkerforgeException {
        invokeMethodOf(actions).brickletRealTimeClockV2SetAlarm(month, day, hour, minute, second, weekday, interval);
    }


    @Override
    @RuleAction(label = "Get Alarm")
    public @ActionOutput(name = "month", type="int")
           @ActionOutput(name = "day", type="int")
           @ActionOutput(name = "hour", type="int")
           @ActionOutput(name = "minute", type="int")
           @ActionOutput(name = "second", type="int")
           @ActionOutput(name = "weekday", type="int")
           @ActionOutput(name = "interval", type="int")
           Map<String, Object> brickletRealTimeClockV2GetAlarm(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClockV2.Alarm value = dev.getAlarm();
        
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("weekday", value.weekday);
        result.put("interval", value.interval);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetAlarm(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetAlarm();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletRealTimeClockV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2ReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletRealTimeClockV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletRealTimeClockV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletRealTimeClockV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClockV2.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletRealTimeClockV2Reset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletRealTimeClockV2Reset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletRealTimeClockV2Reset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletRealTimeClockV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletRealTimeClockV2Wrapper dev = ((@Nullable BrickletRealTimeClockV2Wrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletRealTimeClockV2.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletRealTimeClockV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletRealTimeClockV2GetIdentity();
    }
}
