/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.ThingStatus;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.binding.BaseThingHandler;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.openhab.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.thing.CommonTriggerEvents;
import java.time.ZonedDateTime;
import java.time.ZoneId;
import org.eclipse.smarthome.core.library.types.DateTimeType;

import com.tinkerforge.BrickletRealTimeClock;
import com.tinkerforge.IPConnection;
import com.tinkerforge.TinkerforgeException;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;

/**
 * Wraps the Java Bindings of the Real-Time Clock Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@NonNullByDefault
public class BrickletRealTimeClockWrapper extends BrickletRealTimeClock implements StandardFlashable, DeviceWrapper {
    public static final DeviceInfo DEVICE_INFO = new DeviceInfo(BrickletRealTimeClock.DEVICE_DISPLAY_NAME, "brickletrealtimeclock", BrickletRealTimeClock.DEVICE_IDENTIFIER, BrickletRealTimeClockWrapper.class, BrickletRealTimeClockActions.class, "2.0.1", false);

    public BrickletRealTimeClockWrapper(String uid, IPConnection ipcon) {
        super(uid, ipcon);
    }

    private List<ScheduledFuture<?>> manualChannelUpdates = new ArrayList<>();
    private List<ListenerReg<?>> listenerRegs = new ArrayList<>();

    public void cancelManualUpdates() {
        manualChannelUpdates.forEach(f -> f.cancel(true));
    }

    public <T> T reg(T listener, Consumer<T> toRemove) {
        listenerRegs.add(new ListenerReg<T>(listener, toRemove));
        return listener;
    }

    
    private final Logger logger = LoggerFactory.getLogger(BrickletRealTimeClockWrapper.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn, ScheduledExecutorService scheduler, BaseThingHandler handler) throws TinkerforgeException {
        this.setResponseExpectedAll(true);
        BrickletRealTimeClockConfig cfg = (BrickletRealTimeClockConfig) config.as(BrickletRealTimeClockConfig.class);
        this.addDateTimeListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {updateStateFn.accept("BrickletRealTimeClockDateTime", transformBrickletRealTimeClockDateTimeCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeDateTimeListener));
		this.addDateTimeListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {updateStateFn.accept("BrickletRealTimeClockTimestamp", transformBrickletRealTimeClockTimestampCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeDateTimeListener));
		this.addAlarmListener(this.reg((year, month, day, hour, minute, second, centisecond, weekday, timestamp) -> {if(true) {triggerChannelFn.accept("BrickletRealTimeClockAlarm", transformBrickletRealTimeClockAlarmCallback0(year, month, day, hour, minute, second, centisecond, weekday, timestamp, cfg));}}, this::removeAlarmListener));
        this.setDateTimeCallbackPeriod(cfg.dateTimeUpdateInterval);
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        listenerRegs.forEach(ListenerReg::deregister);

        
        this.setDateTimeCallbackPeriod(0);
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        List<String> result = new ArrayList<String>();
        result.add("BrickletRealTimeClockDateTime");
		result.add("BrickletRealTimeClockTimestamp");
		result.add("BrickletRealTimeClockAlarm");
        return result;
    }

    public static @Nullable ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "BrickletRealTimeClockDateTime":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockDateTime"), "Date Time", "DateTime").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRealTimeClockDateTime")).withDescription("The real-time clock handles leap year and inserts the 29th of February accordingly. But leap seconds are not handled. The time is stored as UTC on the clock and converted into your system's timezone when accessed by OpenHAB.").build();
            case "BrickletRealTimeClockTimestamp":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockTimestamp"), "Timestamp", "Number:Time").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:BrickletRealTimeClockTimestamp")).withDescription("the current date and the time of the real-time clock converted to seconds. The timestamp has an effective resolution of hundredths of a second.")
.withStateDescription(StateDescriptionFragmentBuilder.create().withPattern("%.3f %unit%").withReadOnly(true).build().toStateDescription()).build();
default:
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
             Map<String, String> thingTypeProperties = ThingTypeBuilder.instance(thingTypeUID, "unused").build().getProperties();
             Map<String, String> props = new HashMap<String, String>(thingTypeProperties);
             props.putIfAbsent(TinkerforgeBindingConstants.PROPERTY_MINIMUM_FIRMWARE_VERSION, DEVICE_INFO.minimumFWVersion);
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Real-Time Clock Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.toString())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Battery-backed real-time clock").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("BrickletRealTimeClockDateTime", new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockDateTime")).withLabel("Date Time").build(), new ChannelDefinitionBuilder("BrickletRealTimeClockTimestamp", new ChannelTypeUID("tinkerforge", "BrickletRealTimeClockTimestamp")).withLabel("Timestamp").build(), new ChannelDefinitionBuilder("BrickletRealTimeClockAlarm", new ChannelTypeUID("system", "trigger")).withLabel("Alarm").withDescription("This listener is triggered every time the current date and time matches the configured alarm (see the setAlarm action).").build())).withProperties(props).build();
    }

    public static @Nullable ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:brickletrealtimeclock":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList(ConfigDescriptionParameterBuilder.create("dateTimeUpdateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval for the date time and timestamp in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Date Time Update Interval").withUnit("ms").withMinimum(BigDecimal.valueOf(0)).withMaximum(BigDecimal.valueOf(4294967295L)).build())).withParameterGroups(Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals"))).build();
            case "channel-type:tinkerforge:BrickletRealTimeClockDateTime":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            case "channel-type:tinkerforge:BrickletRealTimeClockTimestamp":
                return ConfigDescriptionBuilder.create(uri).withParameters(Arrays.asList()).withParameterGroups(Arrays.asList()).build();
            default:
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String channel, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletRealTimeClockConfig cfg = (BrickletRealTimeClockConfig) config.as(BrickletRealTimeClockConfig.class);
switch(channel) {
            case "BrickletRealTimeClockDateTime":
                   {
            BrickletRealTimeClock.DateTime value = this.getDateTime();
            if(true)
                updateStateFn.accept(channel, transformBrickletRealTimeClockDateTimeGetter0(value, cfg));
        }
                   return;
            case "BrickletRealTimeClockTimestamp":
                   {
            long value = this.getTimestamp();
            if(true)
                updateStateFn.accept(channel, transformBrickletRealTimeClockTimestampGetter0(value, cfg));
        }
                   return;
            case "BrickletRealTimeClockAlarm":
               return;
default:
                logger.warn("Refresh for unknown channel {}", channel);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        
        switch(channel) {
            case "BrickletRealTimeClockDateTime":
                if (command instanceof DateTimeType) {
                    DateTimeType cmd = (DateTimeType) command;
this.setDateTime(cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getYear(), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getMonth().getValue(), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getDayOfMonth(), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getHour(), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getMinute(), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getSecond(), (short)(cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getNano() / 1000 / 1000 / 10), (short)cmd.getZonedDateTime().withZoneSameInstant(ZoneId.of("UTC")).getDayOfWeek().getValue());
                }
        
                else {
                    logger.warn("Command type {} not supported for channel {}. Please use one of DateTimeType.", command.getClass().getName(), channel);
                }
                
                break;
default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockDateTimeCallback0(int year, short month, short day, short hour, short minute, short second, short centisecond, short weekday, long timestamp, BrickletRealTimeClockConfig cfg) {
        return new DateTimeType(ZonedDateTime.of(year, month, day, hour, minute, second, centisecond * 10 * 1000 * 1000, ZoneId.of("UTC")).withZoneSameInstant(ZoneId.systemDefault()));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockTimestampCallback0(int year, short month, short day, short hour, short minute, short second, short centisecond, short weekday, long timestamp, BrickletRealTimeClockConfig cfg) {
        return new QuantityType<>(timestamp / 1000.0, SmartHomeUnits.SECOND);
    }
	    private String transformBrickletRealTimeClockAlarmCallback0(int year, short month, short day, short hour, short minute, short second, short centisecond, short weekday, long timestamp, BrickletRealTimeClockConfig cfg) {
        return "";
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockDateTimeGetter0(BrickletRealTimeClock.DateTime value, BrickletRealTimeClockConfig cfg) {
        return new DateTimeType(ZonedDateTime.of(value.year, value.month, value.day, value.hour, value.minute, value.second, value.centisecond * 10 * 1000 * 1000, ZoneId.of("UTC")).withZoneSameInstant(ZoneId.systemDefault()));
    }
	    private org.eclipse.smarthome.core.types.State transformBrickletRealTimeClockTimestampGetter0(long value, BrickletRealTimeClockConfig cfg) {
        return new QuantityType<>(value / 1000.0, SmartHomeUnits.SECOND);
    }

}