/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletSoundPressureLevel;

/**
 * Passes through actions of the Sound Pressure Level Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSoundPressureLevelActions implements ThingActions, IBrickletSoundPressureLevelActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletSoundPressureLevelActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletSoundPressureLevelActions.class.getName())) {
            if (actions instanceof IBrickletSoundPressureLevelActions) {
                return (IBrickletSoundPressureLevelActions) actions;
            } else {
                return (IBrickletSoundPressureLevelActions) Proxy.newProxyInstance(IBrickletSoundPressureLevelActions.class.getClassLoader(),
                        new Class[] { IBrickletSoundPressureLevelActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletSoundPressureLevelActions");
    }

    
    @Override
    @RuleAction(label = "Get Decibel")
    public @ActionOutput(name = "decibel", type="int")
           Map<String, Object> brickletSoundPressureLevelGetDecibel(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getDecibel();
        
        result.put("decibel", value);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetDecibel(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetDecibel();
    }


    @Override
    @RuleAction(label = "Get Spectrum")
    public @ActionOutput(name = "spectrum", type="int[]")
           Map<String, Object> brickletSoundPressureLevelGetSpectrum(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int[] value = dev.getSpectrum();
        
        result.put("spectrum", value);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetSpectrum(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetSpectrum();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "fftSize", type="int")
           @ActionOutput(name = "weighting", type="int")
           Map<String, Object> brickletSoundPressureLevelGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletSoundPressureLevel.Configuration value = dev.getConfiguration();
        
        result.put("fftSize", value.fftSize);
        result.put("weighting", value.weighting);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetConfiguration();
    }


    @Override
    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletSoundPressureLevelReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        long value = dev.readUID();
        
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelReadUID();
    }


    @Override
    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletSoundPressureLevelGetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getChipTemperature();
        
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetChipTemperature();
    }


    @Override
    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletSoundPressureLevelGetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getStatusLEDConfig();
        
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetStatusLEDConfig();
    }


    @Override
    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletSoundPressureLevelGetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletSoundPressureLevel.SPITFPErrorCount value = dev.getSPITFPErrorCount();
        
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetSPITFPErrorCount();
    }

    @Override
    @RuleAction(label = "Reset")
    public void brickletSoundPressureLevelReset(
            ) throws TinkerforgeException {
        DeviceHandler h = handler;
        if(h == null) {
            return;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return;
        }
        dev.reset();
        
    }

    public static void brickletSoundPressureLevelReset(@Nullable ThingActions actions) throws TinkerforgeException {
        invokeMethodOf(actions).brickletSoundPressureLevelReset();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletSoundPressureLevelGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletSoundPressureLevelWrapper dev = ((@Nullable BrickletSoundPressureLevelWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletSoundPressureLevel.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletSoundPressureLevelGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletSoundPressureLevelGetIdentity();
    }
}
