/**
 * Copyright (c) 2010-2020 Contributors to the openHAB project
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */

 /* ***********************************************************
 * This file was automatically generated on 2022-01-03.      *
 *                                                           *
 * openHAB Bindings Version 2.0.0                            *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package org.openhab.binding.tinkerforge.internal.device;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.jdt.annotation.NonNull;
import org.openhab.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.eclipse.smarthome.core.types.RefreshType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import java.util.Map;
import java.util.HashMap;

import com.tinkerforge.TinkerforgeException;
import com.tinkerforge.BrickletVoltageCurrent;

/**
 * Passes through actions of the Voltage/Current Bricklet.
 * @author Erik Fleckstein - Initial contribution
 */
@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentActions implements ThingActions, IBrickletVoltageCurrentActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

    private static IBrickletVoltageCurrentActions invokeMethodOf(@Nullable ThingActions actions) {
        if (actions == null) {
            throw new IllegalArgumentException("actions cannot be null");
        }
        if (actions.getClass().getName().equals(BrickletVoltageCurrentActions.class.getName())) {
            if (actions instanceof IBrickletVoltageCurrentActions) {
                return (IBrickletVoltageCurrentActions) actions;
            } else {
                return (IBrickletVoltageCurrentActions) Proxy.newProxyInstance(IBrickletVoltageCurrentActions.class.getClassLoader(),
                        new Class[] { IBrickletVoltageCurrentActions.class }, (Object proxy, Method method, Object[] args) -> {
                            Method m = actions.getClass().getDeclaredMethod(method.getName(),
                                    method.getParameterTypes());
                            return m.invoke(actions, args);
                        });
            }
        }
        throw new IllegalArgumentException("Actions is not an instance of BrickletVoltageCurrentActions");
    }

    
    @Override
    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> brickletVoltageCurrentGetVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getVoltage();
        
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetVoltage();
    }


    @Override
    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> brickletVoltageCurrentGetCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getCurrent();
        
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetCurrent();
    }


    @Override
    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type="int")
           Map<String, Object> brickletVoltageCurrentGetPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        int value = dev.getPower();
        
        result.put("power", value);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetPower(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetPower();
    }


    @Override
    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="short")
           @ActionOutput(name = "voltageConversionTime", type="short")
           @ActionOutput(name = "currentConversionTime", type="short")
           Map<String, Object> brickletVoltageCurrentGetConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrent.Configuration value = dev.getConfiguration();
        
        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetConfiguration();
    }


    @Override
    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "gainMultiplier", type="int")
           @ActionOutput(name = "gainDivisor", type="int")
           Map<String, Object> brickletVoltageCurrentGetCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrent.Calibration value = dev.getCalibration();
        
        result.put("gainMultiplier", value.gainMultiplier);
        result.put("gainDivisor", value.gainDivisor);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetCalibration();
    }


    @Override
    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletVoltageCurrentGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        DeviceHandler h = handler;
        if(h == null) {
            return result;
        }
        
        @Nullable BrickletVoltageCurrentWrapper dev = ((@Nullable BrickletVoltageCurrentWrapper)h.getDevice());
        if(dev == null) {
            return result;
        }

        BrickletVoltageCurrent.Identity value = dev.getIdentity();
        
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletVoltageCurrentGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        return invokeMethodOf(actions).brickletVoltageCurrentGetIdentity();
    }
}
